**********************************************************************
	subroutine bsm_defaultparams()
**********************************************************************
*	set default parameters of the THDM
*---------------------------------------------------------------------
*	24.4.16 Stefan Dittmaier
**********************************************************************
	include '../COMMON/param_sesm.inc'
	include '../COMMON/param_thdm.inc'
	include '../COMMON/param_bsm.inc'

        TB   = 2d0
        TA   = -0.2d0 / sqrt(1d0-0.2d0**2)	! sa = -0.2d0
        Mh0  = 125d0
        MHH  = 750d0
        MA0  = 300d0
        MHp  = 300d0

c SESM
        l12  = 1d-2

c THDM
        lam5 = 1d-4

	end

**********************************************************************
	subroutine bsm_setparams()
**********************************************************************
*	set dependent generic BSM parameters
*---------------------------------------------------------------------
*	24.4.16 Stefan Dittmaier
**********************************************************************
	include '../COMMON/param_bsm.inc'
        include '../COMMON/xparam.inc'

c angle beta
        TB2 = TB**2
        SB  = TB/sqrt(1d0+TB2)
        CB  = SB/TB
        CB2 = CB**2
        SB2 = SB**2
        S2B = 2d0*CB*SB
        C2B = CB**2 - SB**2
c angle alpha
        TA2 = TA**2
        SA  = TA/sqrt(1d0+TA2)
	if (ta.eq.0d0) then
	  sa = 0d0
	  ca = 1d0
	elseif (ta.eq.1d30) then
	  sa = 1d0
	  ca = 0d0
	else
          CA  = SA/TA
	endif
        CA2 = CA**2
        SA2 = SA**2
        S2A = 2d0*CA*SA
        C2A = CA**2 - SA**2
c angles alpha and beta
        CBA = CB*CA + SB*SA
        CAB = CB*CA - SB*SA
        SBA = SB*CA - CB*SA
        SAB = SB*CA + CB*SA

c real mass squares
        Mh02 = Mh0**2
        MHH2 = MHH**2
        MA02 = MA0**2
        MHp2 = MHp**2

c complex mass parameters
        xMh0  = Mh0
        xMHH  = MHH
        xMA0  = MA0
        xMHp  = MHp
        xMh02 = xMh0**2
        xMHH2 = xMHH**2
        xMA02 = xMA0**2
        xMHp2 = xMHp**2

	end

**********************************************************************
	subroutine thdm_setcoupl(type,hbos)
**********************************************************************
*	set coupling factors of the THDM
*---------------------------------------------------------------------
*	24.4.16 Stefan Dittmaier
**********************************************************************
        integer type,hbos
	real*8 v2

        include '../COMMON/param.inc'
        include '../COMMON/xparam.inc'
	include '../COMMON/param_thdm.inc'
	include '../COMMON/param_bsm.inc'

c Higgs-V-V couplings relative to the SM
	Ch0VV = SBA
	CHHVV = CBA

c some 3-Higgs couplings
        Ch0h0HH = -(-(2d0*SAB-(3d0*C2A+C2B)*SBA)
     &               *(xMA02+4d0*lam5/el**2*xMW2*xSW2)
     &      +CBA*(-3d0*xMA02*S2A+4d0*xMh02*S2A+2d0*xMHH2*S2A
     &      +xMA02*S2B+4d0*lam5/el**2*xMW2*(-3d0*S2A+S2B)*xSW2))
     &    /(8d0*CB*xMW*SB*xSW)

        Ch0HHHH = (2d0*xMh02*S2A*SBA+4*xMHH2*S2A*SBA
     &      +xMA02*(2d0*CAB-3d0*C2A*CBA+C2B*CBA-3d0*S2A*SBA-S2B*SBA)
     &      +4d0*lam5/el**2*xMW2
     &          *(2*CAB-3d0*C2A*CBA+C2B*CBA-3d0*S2A*SBA-S2B*SBA)*xSW2)
     &    /(8d0*CB*xMW*SB*xSW)

c THDM type 1 
	if (type.eq.1) then
          Cleph0 = CA/SB 
          ClepHH = SA/SB 
          ClepA = 1d0/TB 
          ClepG = 1d0 
          Cuph0 = CA/SB 
          CupHH = SA/SB 
          CupA = 1d0/TB 
          CupG = 1d0 
          Cdoh0 = CA/SB 
          CdoHH = SA/SB 
          CdoA = 1d0/TB 
          CdoG = 1d0
c THDM type 2 
	elseif (type.eq.2) then
          Cleph0 = -SA/CB 
          ClepHH = CA/CB 
          ClepA = -TB 
          ClepG = 1d0 
          Cuph0 = CA/SB 
          CupHH = SA/SB 
          CupA = 1d0/TB 
          CupG = 1d0 
          Cdoh0 = -SA/CB 
          CdoHH = CA/CB 
          CdoA = -TB 
          CdoG = 1d0
c THDM type 3 (lepton specific)
	elseif (type.eq.3) then
          Cleph0 = -SA/CB 
          ClepHH = CA/CB 
          ClepA = -TB 
          ClepG = 1d0 
          Cuph0 = CA/SB 
          CupHH = SA/SB 
          CupA = 1d0/TB 
          CupG = 1d0 
          Cdoh0 = CA/SB 
          CdoHH = SA/SB 
          CdoA = 1d0/TB 
          CdoG = 1d0
c THDM type 4 (flipped)
	elseif (type.eq.4) then
          Cleph0 = CA/SB 
          ClepHH = SA/SB 
          ClepA = 1d0/TB 
          ClepG = 1d0 
          Cuph0 = CA/SB 
          CupHH = SA/SB 
          CupA = 1d0/TB 
          CupG = 1d0 
          Cdoh0 = -SA/CB 
          CdoHH = CA/CB 
          CdoA = -TB 
          CdoG = 1d0
	else
	  write(*,*) 'thdm_setcoupl: THDM of unknown type ',type
          stop
	endif

c process-specific couplings
	if (hbos.eq.1) then
	  CVH   = Ch0VV 
	  CLEPH = Cleph0
	  CUPH  = Cuph0
	  CDOH  = Cdoh0
	elseif (hbos.eq.2) then
	  CVH   = CHHVV 
	  CLEPH = ClepHH
	  CUPH  = CupHH
	  CDOH  = CdoHH
	endif

c check quartic couplings for perturbativity
        lam_np_thdm = 4d0*pi
	v2 = 4d0*MW2/EL**2*SW2
	lam1_thdm = (ca2*MHH2+sa2*Mh02-sb2*(MA02+lam5*v2))/(cb2*v2) 
	lam2_thdm = (sa2*MHH2+ca2*Mh02-cb2*(MA02+lam5*v2) )/(sb2*v2)
	lam3_thdm = s2a/(s2b*v2)*(MHH2-Mh02)-(MA02-2d0*MHP2)/v2-lam5
	lam4_thdm = 2d0*(MA02-MHP2)/v2+lam5
	lam5_thdm = lam5

	end
