**************************************************************
*
*     main program
*
**************************************************************
      implicit none

#ifdef mpiuse
      include "mpif.h"
#endif
      integer iNumProcs, iErr, iMyRank
      common/mpistuff/iNumProcs,iMyRank

      include 'COMMON/processpar.inc'
      include 'COMMON/output.inc'
      integer fs,ii

      character*90 scratchoutputfile
      common/scratchoutputfile/scratchoutputfile

#ifdef mpiuse
      call MPI_Init(iErr)
      call MPI_Comm_size(MPI_COMM_WORLD, iNumProcs, iErr)
      call MPI_Comm_rank(MPI_COMM_WORLD, iMyRank, iErr)
#else
      iMyRank=0
      iNumProcs=1
#endif

CAM create directories for output if they do not already exist
      call system('mkdir -p UNWEIGHTEDEVENTS')
      call system('mkdir -p HISTOGRAMS')
      call system('mkdir -p HISTUNWEIGHTED')

      call init_main    !some options, particle names/properties etc.
      call read_input   !from inputfile

      do fs=1,count_final(1)
        do ii=1,4
	  fname(ii)=fnameinput(ii,fs)
	enddo

        call setup_new_run(fs) !reset the program to its start condition
        call fix_param(0) !derived parameters like alpha (from G_mu) and
                          !technical parameters like slicing cuts
        call check_parameters !check consistency of parameters
        call init_cuts    !phase-space cuts and photon recombination parameters
        call init_hepup   !initialize Les Houches accord common block

        call init_integration
c irrelevant using collier
        call init_fast    !fast evaluation of tensor integrals for qfast=1

        If(iMyRank.EQ.0) Then
           call begin_output(nout,maxout,moreout)
           call begin_output(nout2,maxout,moreout)
        Endif

c        call checkamp   !switch off fast calc. of tensors (fastoff)
c        call checkuvir  !switch off fast calc. of tensors (fastoff)

	call integration

C        call write_multichannel   !output of multichannel integration

      enddo

      If(iMyRank.EQ.0) Then
         If(count_final(1).GT.1) Then
            call summary_output(nout)
            call summary_output(nout2)
         Endif
      Endif

      If(nout.ne.6) close(nout)
      close(nout2,status='DELETE')

#ifdef mpiuse
       call MPI_Finalize(iErr)
#endif

      end

**************************************************************
*
*     read input
*
*     convention: f1,f3:fermions; f2,f4: anti-fermions,
*                 f1+f2: W+/Z-boson, f3+f4: W-/Z-boson
**************************************************************
      subroutine read_input
      implicit none
      include 'COMMON/model.inc'
      include 'COMMON/processpar.inc'
      include 'COMMON/param.inc'
      include 'COMMON/options.inc'
      include 'COMMON/output.inc'
      include 'COMMON/max.inc'
      include 'COMMON/techpar.inc'
      character*100 inputline,storeinput
      character*100 identifier,orgidentifier
      character*200 error_message(maxerr)
      character*60 warning_message(maxerr)
      integer linputline,i1,isep1,isep2,error_count,warning_count
      character*100 findscratchfilechar,findscratchfilechar2
      character*100 hbosonstring
      integer findscratchfile,length,startpos
      integer qrecomb_active,qsoftcoll_active
      data linputline /100/
c output
      character*90 outputfile
      common/outputfile/outputfile
      character*90 scratchoutputfile
      common/scratchoutputfile/scratchoutputfile
c slicing cuts
      real*8 deltaE,deltaTh
      common/slicuts/deltaE,deltaTh
c random generator
      integer rangen,ranshift,randomseed
      common/random/rangen,ranshift,randomseed
c factorization scale
      real*8 fsrfact,nfsrfact
      common/fsrfact/fsrfact,nfsrfact
c cuts
      real*8 ecut(5),scut(5,5),ccut(5,5),precomb(3)
      integer qcuts,fehler
      common/cuts/ecut,scut,ccut,precomb,qcuts

      integer iNumProcs, iErr, iMyRank
      common/mpistuff/iNumProcs,iMyRank

      integer oc
      common/ranluxoutput/oc

c generic BSM parameters
	include 'COMMON/param_bsm.inc'

c THDM parameters
	include 'COMMON/param_thdm.inc'

c SESM parameters
	include 'COMMON/param_sesm.inc'

c BSM input flags
      integer qalphain,qbetain,sgnsba_in
      real*8 cba_in,ba      
      
      qalphain  = 0    ! default: ta given
      qbetain   = 0    ! default: tb given
      sgnsba_in = 0   

       do i1=0,4
         count_final(i1)=0
       enddo
       error_count=0
       warning_count=0
       qrecomb_active=0
       qsoftcoll_active=0

#ifdef mpiuse
       If(iMyRank.EQ.0) Then
#endif

 50    read( *,'((a))',end=51,err=52) inputline
       if(inputline(1:1).eq.'!'.or.inputline(1:1).eq.'#'
     &      .or.inputline(1:1).eq.'') goto 50
       isep1=0
       isep2=linputline+1

       do i1=1,linputline
         if(inputline(i1:i1).eq.'=') then
           isep1=i1
           goto53
         elseif(inputline(i1:i1).eq.'!') then
           isep2=i1
           goto54
         endif
       enddo
 53    continue
       do i1=isep1+1,linputline
         if(inputline(i1:i1).eq.'!') then
           isep2=i1
           goto54
         endif
       enddo
 54    continue

       if(isep1.gt.0)then
         identifier= inputline(1:isep1-1)
         orgidentifier=identifier
         do i1=isep1-1,1,-1
           if (identifier(i1:i1).eq.' ') then
             identifier=identifier(1:i1-1)//identifier(i1+1:linputline)
           elseif(identifier(i1:i1).ge.'A'.and.identifier(i1:i1).le.'Z')
     &         then
             if(identifier(i1:i1).eq.'A') then
               identifier(i1:i1)='a'
             elseif(identifier(i1:i1).eq.'B') then
               identifier(i1:i1)='b'
             elseif(identifier(i1:i1).eq.'C') then
               identifier(i1:i1)='c'
             elseif(identifier(i1:i1).eq.'D') then
               identifier(i1:i1)='d'
             elseif(identifier(i1:i1).eq.'E') then
               identifier(i1:i1)='e'
             elseif(identifier(i1:i1).eq.'F') then
               identifier(i1:i1)='f'
             elseif(identifier(i1:i1).eq.'G') then
               identifier(i1:i1)='g'
             elseif(identifier(i1:i1).eq.'H') then
               identifier(i1:i1)='h'
             elseif(identifier(i1:i1).eq.'I') then
               identifier(i1:i1)='i'
             elseif(identifier(i1:i1).eq.'J') then
               identifier(i1:i1)='j'
             elseif(identifier(i1:i1).eq.'K') then
               identifier(i1:i1)='k'
             elseif(identifier(i1:i1).eq.'L') then
               identifier(i1:i1)='l'
             elseif(identifier(i1:i1).eq.'M') then
               identifier(i1:i1)='m'
             elseif(identifier(i1:i1).eq.'N') then
               identifier(i1:i1)='n'
             elseif(identifier(i1:i1).eq.'O') then
               identifier(i1:i1)='o'
             elseif(identifier(i1:i1).eq.'P') then
               identifier(i1:i1)='p'
             elseif(identifier(i1:i1).eq.'Q') then
               identifier(i1:i1)='q'
             elseif(identifier(i1:i1).eq.'R') then
               identifier(i1:i1)='r'
             elseif(identifier(i1:i1).eq.'S') then
               identifier(i1:i1)='s'
             elseif(identifier(i1:i1).eq.'T') then
               identifier(i1:i1)='t'
             elseif(identifier(i1:i1).eq.'U') then
               identifier(i1:i1)='u'
             elseif(identifier(i1:i1).eq.'V') then
               identifier(i1:i1)='v'
             elseif(identifier(i1:i1).eq.'W') then
               identifier(i1:i1)='w'
             elseif(identifier(i1:i1).eq.'X') then
               identifier(i1:i1)='x'
             elseif(identifier(i1:i1).eq.'Y') then
               identifier(i1:i1)='y'
             elseif(identifier(i1:i1).eq.'Z') then
               identifier(i1:i1)='z'
             endif
           endif
         enddo
         write(storeinput,*) inputline(isep1+1:isep2-1)

         if(identifier.eq.'outputfile') then
           read(storeinput,'(a100)',err=61) outputfile
           read(storeinput,*) outputfile
         elseif(identifier.eq.'model') then
           read(storeinput,'(i100)',err=61) model
           read(storeinput,*) model
	   if(model.ne.0.and.model.ne.1.and.model.ne.2.and.model.ne.4) Then
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'model can be assigned only to values 0,1,2,4'
	   endif
         elseif(identifier.eq.'modeltype') then
           read(storeinput,'(i100)',err=61) modeltype
           read(storeinput,*) modeltype
         elseif(identifier.eq.'hboson') then
           read(storeinput,'(a100)',err=61) hbosonstring
           read(storeinput,*) hbosonstring
	   if(hbosonstring.EQ."h0") then
	      hboson=1
	   else if(hbosonstring.EQ."hh") then
	      hboson=2
	   Endif
         elseif(identifier.eq.'nevents') then
           read(storeinput,'(i100)',err=61) nevents
           read(storeinput,*) nevents
           if(nevents.le.0) then
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'nevents must be larger than 0.'	   	   
	   endif
	 elseif(identifier.eq.'nunwevents') then
           read(storeinput,'(i100)',err=61) nunwevents
           read(storeinput,*) nunwevents
           if(nunwevents.lt.0) then
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'nunwevents must be larger than or equal to 0.'	   	   
	   endif
	 elseif(identifier.eq.'lheoutput') then
           read(storeinput,'(i100)',err=61) lheoutput
           read(storeinput,*) lheoutput
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'lheoutput cannot be changed in the current version of Prophecy.'	   	   
         elseif(identifier.eq.'resonancesinlhe') then
           read(storeinput,'(i100)',err=61) resonancesinlhe
           read(storeinput,*) resonancesinlhe
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'resonancesinlhe cannot be changed in the current version of Prophecy.'	   	   
         elseif(identifier.eq.'contrib') then
           read(storeinput,'(i100)',err=61) contrib
           read(storeinput,*) contrib
	   if(contrib.ne.1.and.contrib.ne.2.and.contrib.ne.3) Then
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'contrib can be assigned only to values 1,2,3'
	   endif
         elseif(identifier.eq.'qqcd') then
           read(storeinput,'(i100)',err=61) qqcd
           read(storeinput,*) qqcd
	   if(qqcd.ne.0.and.qqcd.ne.1.and.qqcd.ne.2) Then
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'qqcd can be assigned only to values 0,1,2'
	   endif
         elseif(identifier.eq.'qsoftcoll') then
           read(storeinput,'(i100)',err=61) qsoftcoll
           read(storeinput,*) qsoftcoll
	   qsoftcoll_active=1
	   if(qsoftcoll.ne.1.and.qsoftcoll.ne.2) Then
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'qsoftcoll can be assigned only to values 1,2'
	   endif
         elseif(identifier.eq.'qalp') then
           read(storeinput,'(i100)',err=61) qalp
           read(storeinput,*) qalp
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'qalp cannot be changed in the current version of Prophecy.'	   	   
         elseif(identifier.eq.'fname(1)') then
           If(count_final(1).LE.maxfs) then
	      count_final(1)=count_final(1)+1
	   else
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'too many channels requested: 100 Max  '
	   Endif
	   read(storeinput,'(a100)',err=61) fnameinput(1,count_final(1))
           read(storeinput,*) fnameinput(1,count_final(1))
	 elseif(identifier.eq.'fname(2)') then
           If(count_final(2).LE.maxfs) then
	      count_final(2)=count_final(2)+1
	   else
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'too many channels requested: 100 Max  '
	   Endif
	   read(storeinput,'(a100)',err=61) fnameinput(2,count_final(2))
           read(storeinput,*) fnameinput(2,count_final(2))
         elseif(identifier.eq.'fname(3)') then
           If(count_final(3).LE.maxfs) then
	      count_final(3)=count_final(3)+1
	   else
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'too many channels requested: 100 Max  '
	   Endif
	   read(storeinput,'(a100)',err=61) fnameinput(3,count_final(3))
           read(storeinput,*) fnameinput(3,count_final(3))
         elseif(identifier.eq.'fname(4)') then
           If(count_final(1).LE.maxfs) Then
	      count_final(4)=count_final(4)+1
	   else
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'too many channels requested: 100 Max  '
	   Endif
	   read(storeinput,'(a100)',err=61) fnameinput(4,count_final(4))
           read(storeinput,*) fnameinput(4,count_final(4))

         elseif(identifier.eq.'channel') then
           If(count_final(0).LE.maxfs) Then
	     count_final(0)=count_final(0)+1
	   else
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'too many channels requested: 100 Max  ',identifier
	   Endif
	   read(storeinput,'(a100)',err=61)
     &          finalstateinput(count_final(0))
         elseif(identifier.eq.'qrecomb') then
           read(storeinput,'(i100)',err=61) qrecomb
           read(storeinput,*) qrecomb
	   qrecomb_active=1
	   if(qrecomb.ne.0.and.qrecomb.ne.1) Then
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'qrecomb can be assigned only to value 0,1.'
	   endif
         elseif(identifier.eq.'qrecombcolle') then
           read(storeinput,'(i100)',err=61) qrecombcolle
           read(storeinput,*) qrecombcolle
	   if(qrecombcolle.ne.0.and.qrecombcolle.ne.1) Then
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'qrecombcolle can be assigned only to value 0,1.'
	   endif
         elseif(identifier.eq.'invrecomb') then
           read(storeinput,'(f50.50)',err=61) precomb(1)
           read(storeinput,*) precomb(1)
         elseif(identifier.eq.'precomb(1)') then
           read(storeinput,'(f50.50)',err=61) precomb(1)
           read(storeinput,*) precomb(1)
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'precomb(1) cannot be changed in the current version of Prophecy.'	   	   
         elseif(identifier.eq.'mh') then
           read(storeinput,'(f50.50)',err=61) mhi
           read(storeinput,*) mhi
         elseif(identifier.eq.'1/alpha0') then
           read(storeinput,'(f50.50)',err=61) alpha0
           read(storeinput,*) alpha0
           alpha0 = 1d0/alpha0
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'alpha0 is not used and cannot be changed in the current version of Prophecy.'	   	   
         elseif(identifier.eq.'alphas') then
           read(storeinput,'(f50.50)',err=61) alphas
           read(storeinput,*) alphas
         elseif(identifier.eq.'gf') then
           read(storeinput,'(f50.50)',err=61) gf
           read(storeinput,*) gf
         elseif(identifier.eq.'mz') then
           read(storeinput,'(f50.50)',err=61) mzi
           read(storeinput,*) mzi
         elseif(identifier.eq.'mw') then
           read(storeinput,'(f50.50)',err=61) mwi
           read(storeinput,*) mwi
         elseif(identifier.eq.'gammaz') then
           read(storeinput,'(f50.50)',err=61) gzi
           read(storeinput,*) gzi
         elseif(identifier.eq.'gammaw') then
           read(storeinput,'(f50.50)',err=61) gwi
           read(storeinput,*) gwi
         elseif(identifier.eq.'me') then
           read(storeinput,'(f50.50)',err=61) ml(1)
           read(storeinput,*) ml(1)
         elseif(identifier.eq.'mmu') then
           read(storeinput,'(f50.50)',err=61) ml(2)
           read(storeinput,*) ml(2)
         elseif(identifier.eq.'mtau') then
           read(storeinput,'(f50.50)',err=61) ml(3)
           read(storeinput,*) ml(3)
         elseif(identifier.eq.'md') then
           read(storeinput,'(f50.50)',err=61) mqm(1)
           read(storeinput,*) mqm(1)
         elseif(identifier.eq.'mu') then
           read(storeinput,'(f50.50)',err=61) mqp(1)
           read(storeinput,*) mqp(1)
         elseif(identifier.eq.'ms') then
           read(storeinput,'(f50.50)',err=61) mqm(2)
           read(storeinput,*) mqm(2)
         elseif(identifier.eq.'mc') then
           read(storeinput,'(f50.50)',err=61) mqp(2)
           read(storeinput,*) mqp(2)
         elseif(identifier.eq.'mb') then
           read(storeinput,'(f50.50)',err=61) mqm(3)
           read(storeinput,*) mqm(3)
         elseif(identifier.eq.'mt') then
           read(storeinput,'(f50.50)',err=61) mqp(3)
           read(storeinput,*) mqp(3)

         elseif(identifier.eq.'qllfsr') then
           read(storeinput,'(i100)',err=61) qllfsr
           read(storeinput,*) qllfsr
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'qllfsr cannot be changed in the current version of Prophecy.'	   	   
         elseif(identifier.eq.'deltae') then
           read(storeinput,'(f50.50)',err=61) deltae
           read(storeinput,*) deltae
         elseif(identifier.eq.'deltath') then
           read(storeinput,'(f50.50)',err=61) deltath
           read(storeinput,*) deltath
         elseif(identifier.eq.'ranshift') then
           read(storeinput,'(i100)',err=61) ranshift
           read(storeinput,*) ranshift
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'ranshift cannot be changed in the current version of Prophecy.'	   	   
         elseif(identifier.eq.'randomseed') then
           read(storeinput,'(i100)',err=61) randomseed
           read(storeinput,*) randomseed
	   If(randomseed.NE.-1) rangen=3
           if(randomseed.lt.-1) then
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'randomseed must be larger than -1.'	   	   
	   endif
         elseif(identifier.eq.'qhist') then
           read(storeinput,'(i100)',err=61) qhist
           read(storeinput,*) qhist
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'qhist cannot be changed in the current version of Prophecy.'	   	   
         elseif(identifier.eq.'moreout') then
           read(storeinput,'(i100)',err=61) moreout
           read(storeinput,*) moreout
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'moreout cannot be changed in the current version of Prophecy.'	   	   
         elseif(identifier.eq.'massmapping') then
           read(storeinput,'(f50.50)',err=61) massmapping
           read(storeinput,*) massmapping
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'massmapping cannot be changed in the current version of Prophecy.'	   	   
         elseif(identifier.eq.'mininvmass') then
           read(storeinput,'(f50.50)',err=61) mininvmass
           read(storeinput,*) mininvmass
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'mininvmass cannot be changed in the current version of Prophecy.'	   	   
         elseif(identifier.eq.'mffcutrel') then
           read(storeinput,'(f50.50)',err=61) mffcutrel
           read(storeinput,*) mffcutrel
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'mffcutrel cannot be changed in the current version of Prophecy.'	   	   
         elseif(identifier.eq.'nfsrfact') then
           read(storeinput,'(f50.50)',err=61) nfsrfact
           read(storeinput,*) nfsrfact
              If(error_count.LT.maxerr) error_count=error_count+1
	      write(error_message(error_count),*)
     &           'nfsrfact cannot be changed in the current version of Prophecy.'	   	   
         elseif(identifier.eq.'qsm4') then
           read(storeinput,'(i100)',err=61) qsm4
           read(storeinput,*) qsm4
         elseif(identifier.eq.'ml4') then
           read(storeinput,'(f50.50)',err=61) ml4
           read(storeinput,*) ml4
         elseif(identifier.eq.'mn4') then
           read(storeinput,'(f50.50)',err=61) mn4
           read(storeinput,*) mn4
         elseif(identifier.eq.'md4') then
           read(storeinput,'(f50.50)',err=61) md4
           read(storeinput,*) md4
         elseif(identifier.eq.'mu4') then
           read(storeinput,'(f50.50)',err=61) mu4
           read(storeinput,*) mu4
         elseif(identifier.eq.'renscheme') then
           read(storeinput,'(i100)',err=61) renscheme
           read(storeinput,*) renscheme
         elseif(identifier.eq.'mrenbsm1') then
           read(storeinput,'(f50.50)',err=61) mrenbsm1
           read(storeinput,*) mrenbsm1
         elseif(identifier.eq.'mrenbsm2') then
           read(storeinput,'(f50.50)',err=61) mrenbsm2
           read(storeinput,*) mrenbsm2
         elseif(identifier.eq.'tb') then
           read(storeinput,'(f50.50)',err=61) tb_in
           read(storeinput,*) tb_in
           if (qbetain.eq.0) then
             qbetain = 1
           else
              If(error_count.LT.maxerr) error_count=error_count+1
              write(error_message(error_count),*) 
     &       'angle beta already set. Change input file!'
           endif
	   If(tb_in.le.0D0) Then
             If(error_count.LT.maxerr) error_count=error_count+1
 	     write(error_message(error_count),*) 
     &       'tb can only take values tb > 0D0!'
	   Endif
         elseif(identifier.eq.'sb') then
           read(storeinput,'(f50.50)',err=61) sb_in
           read(storeinput,*) sb_in
           if (qbetain.eq.0) then
             qbetain = 2
           else
              If(error_count.LT.maxerr) error_count=error_count+1
              write(error_message(error_count),*) 
     &       'angle beta already set. Change input file!'
           endif
	   If(sb_in.ge.1D0 .or. sb_in.le.0D0) Then
             If(error_count.LT.maxerr) error_count=error_count+1
 	     write(error_message(error_count),*) 
     &       'sb can only take values 0D0 < sb < 1D0!'
	   Endif
         elseif(identifier.eq.'cb') then
           read(storeinput,'(f50.50)',err=61) cb_in
           read(storeinput,*) cb_in
           if (qbetain.eq.0) then
             qbetain = 3
           else
              If(error_count.LT.maxerr) error_count=error_count+1
              write(error_message(error_count),*) 
     &       'angle beta already set. Change input file!'
           endif
	   If(cb_in.ge.1D0 .or. cb_in.le.0D0) Then
             If(error_count.LT.maxerr) error_count=error_count+1
 	     write(error_message(error_count),*) 
     &       'cb can only take values 0D0 < cb < 1D0!'
	   Endif
         elseif(identifier.eq.'ta') then
           read(storeinput,'(f50.50)',err=61) ta_in
           read(storeinput,*) ta_in
           if (qalphain.eq.0) then
             qalphain = 1
           else
              If(error_count.LT.maxerr) error_count=error_count+1
 	     write(error_message(error_count),*) 
     &       'angle alpha already set. Change input file!'
           endif
         elseif(identifier.eq.'sa') then
           read(storeinput,'(f50.50)',err=61) sa_in
           read(storeinput,*) sa_in
           if (qalphain.eq.0) then
             qalphain = 2
           else
              If(error_count.LT.maxerr) error_count=error_count+1
 	     write(error_message(error_count),*) 
     &       'angle alpha already set. Change input file!'
           endif
	   If(sa_in.gt.1D0 .or. sa_in.le.-1D0) Then
             If(error_count.LT.maxerr) error_count=error_count+1
 	     write(error_message(error_count),*) 
     &       'sa can only take values -1D0 < sa < 1D0!'
	   Endif
         elseif(identifier.eq.'cba') then
           read(storeinput,'(f50.50)',err=61) cba_in
           read(storeinput,*) cba_in
           if (qalphain.eq.0) then
             qalphain = 3
           else
              If(error_count.LT.maxerr) error_count=error_count+1
 	     write(error_message(error_count),*) 
     &       'angle alpha already set. Change input file!'
           endif
	   If(cba_in.gt.1D0 .or. cba_in.lt.-1D0) Then
             If(error_count.LT.maxerr) error_count=error_count+1
 	     write(error_message(error_count),*) 
     &       'cba can only take values -1D0 <= cba <= 1D0!'
	   Endif
         elseif(identifier.eq.'sgnsba') then
           read(storeinput,'(i100)',err=61) sgnsba_in
           read(storeinput,*) sgnsba_in
           if (abs(sgnsba_in).ne.1) then
              If(error_count.LT.maxerr) error_count=error_count+1
 	     write(error_message(error_count),*) 
     &       'sgn(sba) already set. Change input file!'
           endif
         elseif(identifier.eq.'mh0') then
           read(storeinput,'(f50.50)',err=61) mh0
           read(storeinput,*) mh0
         elseif(identifier.eq.'mhh') then
           read(storeinput,'(f50.50)',err=61) mhh
           read(storeinput,*) mhh
         elseif(identifier.eq.'ma0') then
           read(storeinput,'(f50.50)',err=61) ma0
           read(storeinput,*) ma0
         elseif(identifier.eq.'mhp') then
           read(storeinput,'(f50.50)',err=61) mhp
           read(storeinput,*) mhp
         elseif(identifier.eq.'lam5') then
           read(storeinput,'(f50.50)',err=61) lam5
           read(storeinput,*) lam5
           lam5_in = lam5
         elseif(identifier.eq.'l12') then
           read(storeinput,'(f50.50)',err=61) l12
           read(storeinput,*) l12
           l12_in = l12
         elseif(identifier.ne.' ') then
           If(error_count.LT.maxerr) error_count=error_count+1
	   write(error_message(error_count),*)
     &           'identifier not known:  ',identifier
           If(error_count.LT.maxerr) error_count=error_count+1
	   write(error_message(error_count),*)
     &           'identifier not known:  ',orgidentifier
         endif
         goto 50
       endif

 52    If(error_count.LT.maxerr) error_count=error_count+1
       write(error_message(error_count),*)
     &       'input not known: ',inputline
       goto 50
 61    If(error_count.LT.maxerr) error_count=error_count+1
       write(error_message(error_count),*)
     &       'input not appropriate for identifier ''',
     &       identifier(1:isep1-1),''' : ',storeinput
       goto 50

 51    continue       

c determine Higgs mixing angles (ta_in, tb_in) from input
c SESM
       if (model.eq.1) then
         if ((qalphain.eq.0).or.(qalphain.eq.1)) then   ! ta = input
         elseif (qalphain.eq.2) then                    ! sa = input
           sa = sa_in
           ca = sqrt(1d0-sa**2)
           if (sa.eq.1d0) then
             ta_in = 1d30
	   else
             ta_in = sa/ca
	   endif
         endif
         if (sa*l12_in.lt.0d0) then
              If(error_count.LT.maxerr) error_count=error_count+1
            write(error_message(error_count),*)
     &           'sin(alpha) and lambda_12 cannot have opposite signs.' 
            write(error_message(error_count),*) 'Change input file!'
         endif
       endif
c THDM
       if (model.eq.2) then
         if ((qbetain.eq.0).or.(qbetain.eq.1)) then     ! tb = input
         elseif (qbetain.eq.2) then                     ! sb = input
           sb = sb_in
           cb = sqrt(1d0-sb**2)
           tb_in = sb/cb
         elseif (qbetain.eq.3) then                     ! cb = input
           cb = cb_in
           sb = sqrt(1d0-cb**2)
           tb_in = sb/cb
         endif
         if ((qalphain.eq.0).or.(qalphain.eq.1)) then   ! ta = input
         elseif (qalphain.eq.2) then                    ! sa = input
           sa = sa_in
           if (sa.ne.1d0) then
             ca = sqrt(1d0-sa**2)
             ta_in = sa/ca
	   else
             ca = 0d0
             ta_in = 1d30
	   endif
         elseif (qalphain.eq.3) then                    ! cba = input
           if (abs(sgnsba_in).ne.1) then
              if(error_count.LT.maxerr) error_count=error_count+1
 	      write(error_message(error_count),*) 
     &          'Sign of sba not correctly set. Change input!'
           endif
           cba = cba_in
           sba = sqrt(1d0-cba**2)*sgnsba_in
           ba  = log(dcmplx(cba,sba))/dcmplx(0d0,1d0) 
           if (abs(datan(tb_in)-ba).gt.datan(1d0)*2d0) then
              if(error_count.LT.maxerr) error_count=error_count+1
 	      write(error_message(error_count),*) 
     &          'Sign of sba inconsistent. Change sign!'
           endif
           ta_in = tan(datan(tb_in)-ba)
         endif
       endif
       sb_in = tb_in/sqrt(1d0+tb_in**2)
       cb_in = sb_in/tb_in
       if (ta_in.ne.1d30) then
         sa_in = ta_in/sqrt(1d0+ta_in**2)
         ca_in = sa_in/ta_in
       else
         sa_in = 1d0
         ca_in = 0d0
       endif

#ifdef mpiuse
       Endif

       call mpi_input_comm(
     &  outputfile,nevents,nunwevents,contrib,qqcd,qalp,
     &  count_final,fnameinput,finalstateinput,qrecomb,qllfsr,ranshift,
     &  randomseed,
     &  mhi,alpha0,alphas,gf,mzi,mwi,gzi,gwi,ml,mqm,mqp,precomb,
     &  deltae,deltath,massmapping,mininvmass,nfsrfact,
     &  qsoftcoll,qhist,moreout,qrecombcolle,lheoutput,resonancesinlhe,
     &  model,modeltype,hboson,mffcutrel,qsm4,ml4,mn4,md4,mu4,
     &  renscheme,mrenbsm1,mrenbsm2,tb,tb_in,ta,ta_in,sa,ca,
     &  mh0,mhh,ma0,mhp,lam5,lam5_in,l12,l12_in)

#endif

      open(unit=nout,file=outputfile,status='unknown',iostat=fehler)
      if (fehler.ne.0) then
         close(nout)
	 outputfile='plot'
         nout=6
      endif
      oc=nout      
      findscratchfile=1
      do while(findscratchfile.ne.0)
        Write(findscratchfilechar,*) findscratchfile
	Write(findscratchfilechar2,*) iMyRank
        scratchoutputfile='__'//
     &  findscratchfilechar2(startpos(findscratchfilechar2):length(findscratchfilechar2))//
     &  '_outputscratchfile'//
     &  findscratchfilechar(startpos(findscratchfilechar):length(findscratchfilechar))//'__'
	open(unit=nout2,file=scratchoutputfile,status='new',iostat=fehler)
        findscratchfile=findscratchfile+1
        If(fehler.eq.0) findscratchfile=0
      enddo
      If(iMyRank.EQ.0) Then
        write(nout,*)""
        write(nout,*)
     &        '======================================================='
        write(nout,*)' This is Prophecy4f, version 3.0.2              '
        write(nout,*)' authors: A. Denner, S. Dittmaier, A. Mueck     '
        write(nout,*)
     &        '======================================================='
        write(nout2,*)""
        write(nout2,*)
     &        '======================================================='
        write(nout2,*)' This is Prophecy4f, version 3.0.2             '
        write(nout2,*)' authors: A. Denner, S. Dittmaier, A. Mueck     '
        write(nout2,*)
     &        '======================================================='
#ifdef mpiuse
        If(iMyRank.EQ.0) Then
          write(nout,*) '(running on ',iNumProcs,'cores)'
          write(nout2,*) '(running on ',iNumProcs,'cores)'
        Endif
#endif
        write(nout,*)""
        write(nout2,*)""

        call test_and_transform_input(model,hboson,contrib,qsm4,
     &                nunwevents,qsoftcoll,qrecomb,renscheme,modeltype,
     &                hbosonstring,qrecomb_active,qsoftcoll_active,
     &                error_count,error_message,warning_count,warning_message)

        If(error_count.GT.0) Then
          write(nout,*) 'There have been errors in the input file:'
	  write(nout,*) ''
	  If(nout.ne.6) Then
            write(6,*) 'There have been errors in the input file:'
	    write(6,*) ''
	  Endif
	  do i1=1,error_count
             write(nout,*) error_message(i1)
	    If(nout.ne.6) Then
              write(6,*) error_message(i1)
	    Endif
	  enddo
	  write(nout,*) 'Please check the input.'
	  write(nout,*) 'The Programs stops here.'
	  If(nout.ne.6) Then
	    write(6,*) 'Please check the input.'
	    write(6,*) 'The Programs stops here.'
	  Endif
	  close(nout)
	  Stop
        Endif

        If(warning_count.GT.0) Then
	  do i1=1,warning_count
             write(nout,*) warning_message(i1)
	     write(nout,*) ''
	  enddo
        Endif

      Endif

      call organize_process !translate input into well defined processes

      end

************************************************************************
*
*     test consistency of the chosen input
*
************************************************************************
      subroutine test_and_transform_input(model,hboson,contrib,qsm4,
     &                nunwevents,qsoftcoll,qrecomb,renscheme,modeltype,
     &                hbosonstring,qrecomb_active,qsoftcoll_active,
     &                error_count,error_message,warning_count,warning_message)
      implicit none

      include 'COMMON/max.inc'
            
      integer model,hboson,contrib,qsm4,nunwevents,qsoftcoll,qrecomb,renscheme
      integer modeltype
      character*100 hbosonstring      
      character*200 error_message(maxerr)
      character*60 warning_message(maxerr)
      integer error_count,warning_count,qrecomb_active,qsoftcoll_active
      
      If(model.eq.0) Then
        If(contrib.ne.1 .AND. contrib.ne.2 .AND. contrib.ne.3) Then
          If(error_count.LT.maxerr) error_count=error_count+1
	  write(error_message(error_count),*)
     &        'The input "contrib" can only be assigned to 1, 2, or 3 in the SM.'	   	   
        Endif
	qsm4=0
      Else If(model.eq.1) Then
        If(contrib.ne.1 .AND. contrib.ne.3) Then
          If(error_count.LT.maxerr) error_count=error_count+1
	  write(error_message(error_count),*)
     &        'The input "contrib" can only be assigned to 1 or 3 in the SESM.'	   	   
        Endif
        If(renscheme.lt.0 .or. renscheme.GT.7) Then
          If(error_count.LT.maxerr) error_count=error_count+1
	  write(error_message(error_count),*)
     &        'The input "renscheme" can only be assigned in the range 1-7 in the SESM.'  
        Endif
        If(hbosonstring.NE."h0".AND.hbosonstring.NE."hh") Then
          If(error_count.LT.maxerr) error_count=error_count+1
	  write(error_message(error_count),*)
     &       'hboson can be assigned only to "h0" and "hh" in the SESM.'	   
        Endif
	qsm4=0
      Else If(model.eq.2) Then
        If(contrib.ne.1 .AND. contrib.ne.3) Then
          If(error_count.LT.maxerr) error_count=error_count+1
	  write(error_message(error_count),*)
     &        'The input "contrib" can only be assigned to 1 or 3 in the THDM.'	   	   
        Endif
        If(renscheme.lt.0 .or. renscheme.GT.8) Then
          If(error_count.LT.maxerr) error_count=error_count+1
	  write(error_message(error_count),*)
     &        'The input "renscheme" can only be assigned in the range 1-8 in the THDM.'	   	   
        Endif
	If(modeltype.lt.1 .or. modeltype.gt.4) Then
          If(error_count.LT.maxerr) error_count=error_count+1
	  write(error_message(error_count),*)
     &       'modeltype can be assigned only to values 1,2,3,4 in the THDM.'
	Endif	
        If(hbosonstring.NE."h0".AND.hbosonstring.NE."hh") Then
          If(error_count.LT.maxerr) error_count=error_count+1
	  write(error_message(error_count),*)
     &       'hboson can be assigned only to "h0" and "hh" in the SESM.'	   
        Endif
	qsm4=0
      Else If(model.eq.4) Then
        If(contrib.ne.1 .AND. contrib.ne.3) Then
          If(error_count.LT.maxerr) error_count=error_count+1
	  write(error_message(error_count),*)
     &        'The input "contrib" can only be assigned to 1 or 3 in the SM4.'	   	   
        Endif
	If(qsm4.ne.1.and.qsm4.ne.2) Then
          If(error_count.LT.maxerr) error_count=error_count+1
	  write(error_message(error_count),*)
     &       'qsm4 can be assigned only to values 1,2 in the SM4'
	endif
      Endif

      If(nunwevents.GT.0) Then
        If(qsoftcoll.NE.2.AND.qsoftcoll_active.EQ.1) Then
          If(error_count.LT.maxerr) error_count=error_count+1
	  write(error_message(error_count),*)
     &        'The input "qsoftcoll" can only be assigned to 2 for nunwevents>0.'	   	   
	Endif
        If(qsoftcoll.NE.2.AND.qsoftcoll_active.EQ.0) Then
          qsoftcoll=2
	  If(warning_count.LT.maxerr) warning_count=warning_count+1
	  write(warning_message(warning_count),*)
     &        'qsoftcoll is set to 2 for nunwevents>0.'
	Endif
        If(qrecomb.NE.0.AND.qrecomb_active.EQ.1) Then
          If(error_count.LT.maxerr) error_count=error_count+1
	  write(error_message(error_count),*)
     &        'The input "qrecomb" can only be assigned to 0 for nunwevents>0.'	   	   
	Endif
        If(qrecomb.NE.0.AND.qrecomb_active.EQ.0) Then
          qrecomb=0
	  If(warning_count.LT.maxerr) warning_count=warning_count+1
	  write(warning_message(warning_count),*)
     &        'qrecomb is set to 0 for nunwevents>0.' 
	Endif
      Endif
      
      end

************************************************************************
*
*     initialize process etc.
*
************************************************************************
      subroutine init_main
      implicit none
      integer i,i1,i2
      include 'COMMON/param.inc'
      include 'COMMON/model.inc'
      include 'COMMON/processpar.inc'
      include 'COMMON/options.inc'
      include 'COMMON/output.inc'
      include 'COMMON/techpar.inc'
c particle names
      real*8 mass(-17:17),width(-17:17)
      character*8 ename(-7:7),gname(-17:17)
      common/particles/ename,gname,mass,width
c unit
      real*8 unit
      common/unit/unit
c random generator
      integer rangen,ranshift,randomseed
      common/random/rangen,ranshift,randomseed
c smearing
      integer smear
      real*8 smrat
      common/smearing/smrat,smear
c factorization scale
      real*8 fsrfact,nfsrfact
      common/fsrfact/fsrfact,nfsrfact
c output
      character*90 outputfile
      common/outputfile/outputfile
c cuts
      real*8 ecut(5),scut(5,5),ccut(5,5),precomb(3)
      integer qcuts
      common/cuts/ecut,scut,ccut,precomb,qcuts
c slicing cuts
      real*8 deltaE,deltaTh
      common/slicuts/deltaE,deltaTh

#ifdef mpiuse
      include "mpif.h"
#endif
      integer iNumProcs, iErr, iMyRank
      common/mpistuff/iNumProcs,iMyRank

c default model
      model     = 0		! Standard Model
      modeltype = 1
      hboson    = 1

c SM variants: 0=standard model, 1=SM4 1-loop, 2=sm4 including leading 2-loop
c qsm4 is set to qsm4=0 if model .ne. 4 for SM4 
      qsm4=1
c default parameters, can be reset by values given in inputfile
c   default SM parameters
      mhi     = 125d0                 ! Higgs boson mass
      alpha0  = 1d0/137.0359997d0     ! alpha(0)
      alphas  = 0.118d0               ! strong coupling constant
      gf      = 1.1663787d-5          ! Fermi constant
      mzi     = 91.1876d0             ! Z boson mass in pdg scheme
      mwi     = 80.385d0              ! W boson mass in pdg scheme
      gzi     = 2.4952d0              ! Z boson width in pdg scheme
      gwi     = 2.085d0               ! W boson width in pdg scheme
      ml(1)   = 0.510998928d-3        ! electron mass
      ml(2)   = 105.6583715d-3        ! muon mass
      ml(3)   = 1.77682d0             ! tau mass
      mqm(1)  = 0.100d0               ! d-quark mass
      mqp(1)  = 0.100d0               ! u-quark mass
      mqm(2)  = 0.100d0               ! s-quark mass
      mqp(2)  = 1.51d0                ! c-quark mass
      mqm(3)  = 4.92d0                 ! b-quark mass
      mqp(3)  = 172.5d0               ! t-quark mass
c   default fermion masses of the 4th generation
      mn4 = 600d0                     ! gen 4 neutrino mass
      ml4 = 600d0                     ! gen 4 lepton mass
      mu4 = 650d0                     ! gen 4 up quark mass
      md4 = 600d0                     ! gen 4 down quark mass
c   default BSM parameters
      call bsm_defaultparams()
c default options, can be reset via inputfile
      outputfile = ''  ! name of outputfile
      nevents = 10000000         ! number of weighted events
      nunwevents = 0             ! number of unweighted events
c lheoutput=0 (1): don't (do) write unweighted events to .lhe file
      lheoutput = 1
c resonancesinlhe=0 (1): don't (do) write information about resonances
c and helicities in *.lhe files
      resonancesinlhe = 0
c Marcus: contrib=5 and 6 have been removed
      contrib = 1          ! contrib: 1=best 2=IBA 3=Born
                           ! more options: 4=Brems 5=no virt.corr.&endp.
                           !              6=only virt.corr.&endp. 7=FSR
      qsoftcoll = 1        ! qsoftcoll: 1=subtraction, 2=slicing
c--- fermions: e,mu,nue,num,dq,uq,sq,cq,anti-e,...
c--- order: e.g. nue anti-e e anti-nue, uq anti-uq e anti-e, ...
      fname(1) = "e"         ! fermion 1
      fname(2) = "anti-e"    ! fermion 2
      fname(3) = "mu"        ! fermion 3
      fname(4) = "anti-mu"   ! fermion 4
      qrecomb = 1      ! qrecomb: 0= no recombination, 1=photon recombination
      qrecombcolle = 0 ! qrecombcolle: 0= no recombination of strictly collinear photons in slicing
                       ! qrecombcolle: 1= recombination of strictly collinear photons with electrons in slicing
      precomb(1)=5d0   ! recombination condition (photon-fermion invariant mass)
      qqcd = 1         ! qqcd: 0=EW, 1=EW+QCD, 2=QCD, 3=a, 4=b, 5=c, 6=d
      qllfsr = 0       ! qllfsr: 0=no 1=expon 2=no expon 3/4=for IBA(exp/no exp)

c output
      nout=2
      nout2=24
c more detailed output (for moreout=1)
      moreout=0
c maximum number of error messages
      maxout=0
c calculate virtual corr. only every nvirt'th time
      nvirt(1) = 100
      nvirt(2) = 20  !if nvirt(2).lt.nvirt(1), QCD has to be calculated first!
      if (qsoftcoll.eq.2) then
        nvirt(1) = 100
        nvirt(2) = 100
      endif
c ranshift: shift for random numbers (different seed)
      ranshift = 0
c fill histograms or don't (1 or 0)
      qhist = 1
c smearing of histograms
      smear = 0
c      smrat = 5d-1 !# on average 'smrat' of events are distributed over 2 bins
c alpha parametrization scheme
      qalp = 1
c factorization scale for FSR (x*MH)^2
      nfsrfact = 1d0
c fast calculation of loop integrals
c      qfast = 1
c irrelevant when using collier !
	qfast =0
c output in MeV
      unit = 1d3
c random generator (1:from RacoonWW, 2:RCarry, 3: RANLUX)
      rangen = 2
      randomseed = -1
c width scheme
      qprop = 1
c complex-mass scheme
      qcomp = 1
c virtual part of corrections (qreal=0:with virtual part, qreal=1:neglect)
      qreal = 0
c bosonic and fermionc contributions to virtual corrections
c not applicable for wwidth and zwidth and corrections to decays in
c NWA because all corrections needed to calculated W/Z width
      qbos  = 1
      qferm = 1
c parameter in h-function (mapping)
      massmapping = 1d-6
c cut for real corr.(min.*CMenergy)^2
      mininvmass = 1d-4
c invariant mass cut for ffbar pairs in 4-particle phase space
c cut is (mffcutrel*CMenergy)^2
      mffcutrel = 1d-3
c cut for nlo/lo weight ratio, events with larger ratios are discarded
      weightcut = 1d6
c debugging output
      debug = 0
c slicing cuts
      deltaE  = 4d-4
      deltaTh = 3d-2
c worst tolerated unweighting inefficiency
      maxunweighteff = 500000

c generic particle names for phase-space generator
c different names than in RacoonWW and CofferGaGa
      gname(0)='XXX'
      gname(1)='e'
      gname(2)='mu'
      gname(3)='tau'
      gname(4)='nue'
      gname(5)='num'
      gname(6)='nut'
      gname(7)='dq'
      gname(8)='sq'
      gname(9)='bq'
      gname(10)='uq'
      gname(11)='cq'
      gname(12)='tq'
      do i = 1,12
        gname(-i) = gname(i)
        gname(-i) = 'anti-'//gname(i)
      enddo
      gname(13)='W-'
      gname(-13)='W+'
      gname(14)='Z0'
      gname(15)='ga'
      gname(16)='H0'
      do i = 14,15
        gname(-i) = gname(i)
      enddo
      gname(17)='   '

c polarizations of fermions and photon in Bremsstrahlungs process
      do i=1,5
        pol(i)=0
      enddo

      end

**************************************************************
*
*     check consistency of floating point behaviour
*
**************************************************************
      subroutine check_fp_testfunc(m2, m)
      implicit none
      include 'COMMON/output.inc'
      real*8 m2,m
      real*8 mm2

      mm2 = m*m
      if (mm2 .ne. m2) then
         write(nout,*)
     &         'Inconsistent floating point behaviour (testfunc).'
         write(nout,*)
     &         'The floating point unit must be set to 64bit mode.'
         close(nout)
	 stop
      endif
      end

      subroutine check_fp_testfunc2(x)
      implicit none
      include 'COMMON/output.inc'
      real*8 x

      if (1d0/x.ne.0.1d0) then
         write(nout,*)
     &         'Inconsistent floating point behaviour (testfunc2).'
         write(nout,*)
     &         'The floating point unit must be set to 64bit mode.'
         Close(nout)
	 stop
      endif
      end

      subroutine check_fp_consistency
      implicit none
      real*8 ma,ma2

      ma = 80.41d0
      ma2 = ma*ma
      call check_fp_testfunc(ma2,ma)
      call check_fp_testfunc2(10d0)
      end

**************************************************************
*
*     check consistency of parameters
*
**************************************************************
      subroutine check_parameters
      implicit none
      include 'COMMON/processpar.inc'
      include 'COMMON/options.inc'
      include 'COMMON/output.inc'
      integer i1,i2,anti,cons
      logical exhist1,exhist2
c particle names
      real*8 mass(-17:17),width(-17:17)
      character*8 ename(-7:7),gname(-17:17)
      common/particles/ename,gname,mass,width
c cuts
      real*8 ecut(5),scut(5,5),ccut(5,5),precomb(3)
      integer qcuts
      common/cuts/ecut,scut,ccut,precomb,qcuts

      call check_fp_consistency()
c inconsistent/unsupported options
      if (nunwevents.gt.0) then
C         write(nout,*)'Unweighting not supported in this version.'
C         close(nout)
C	 stop
      endif
      if (qsoftcoll.ne.1) then
C         write(nout,*)'Slicing not supported in this version.'
C         close(nout)
C	 stop
      endif
      if (qllfsr.ne.0) then
         write(nout,*)'LL FSR not supported in this version.'
         close(nout)
	 stop
      endif
      if (qcuts.ne.0) then
         write(nout,*)'Cuts not supported in this version.'
         close(nout)
	 stop
      endif
      if (nunwevents.gt.0.and.qsoftcoll.eq.1.and.contrib.eq.1) then
         write(nout,*)'Unweighting only supported for slicing method',
     &        ' (qsoftcoll=2) in this version.'
         close(nout)
	 stop
      endif
      if(qrecombcolle.eq.1.and.qsoftcoll.ne.2) then
        write(nout,*)
     &        'Recombination of strictly collinear photons with electrons'//
     &        ' only supported for slicing.'
	Close(nout)
	Stop
      endif
      if (nunwevents.gt.0.AND.(.NOT.(contrib.eq.2.OR.contrib.eq.3))) then
         do i1=1,4
	   do i2=7,12
	     if(fname(i1).EQ.gname(i2)) then
	        write(nout,*)'Unweighting only supported for',
     &        ' leptonic final states in this version.'
	        If(nout.ne.6) Then
		  write(6,*)'Unweighting only supported for',
     &              ' leptonic final states in this version.'
                Endif
		close(nout)
	        stop
	     endif
	   enddo
	 enddo
      endif
      if (nunwevents.gt.0.and.qrecomb.eq.1) then
         write(nout,*)'Recombination for unweighted events can be',
     &        ' done after the run using the event-file'
         close(nout)
	 stop
      endif
      if (nevents.eq.0) then
         write(nout,*)'The number of weighted events can never be 0'
         close(nout)
	 stop
      endif
      if (color.ne.1.and.qsoftcoll.eq.2.and.contrib.eq.1) then
C         write(nout,*)'For semi-leptonic and hadronic final states',
C     &        ' please use the subtraction method (qsoftcoll=1).'
C         close(nout)
C	 stop
      endif
      if (nevents.eq.0) then
         write(nout,*)'The number of weighted events can never be 0'
         close(nout)
	 stop
      endif
      if (nunwevents.gt.0.and.qllfsr.gt.0) then
         write(nout,*)'LL FSR not supported in combination with',
     &        ' unweighted events in this version, qllfsr=',qllfsr,'.'
         close(nout)
	 stop
      endif
      if (contrib.eq.2.and.qllfsr.eq.1.or.qllfsr.eq.2) then
         write(nout,*)'Please use qllfsr=3/4 in combination with IBA',
     &        'otherwise O(alpha) is subtracted.'
         close(nout)
	 stop
      endif
      if(qcomp.eq.1.and.qprop.ne.1)then
        write(nout,*)'Incompatible options: qprop=',qprop,',qcomp=',
     &                                                           qcomp
        close(nout)
	stop
      endif

c consistency checks
c check charge conservation
      if (int(10d0*(q(1)-q(2)+q(3)-q(4))).ne.0d0) then
         write(nout,*)'Check final state, charge is not conserved'
         close(nout)
	 stop
      endif
c check if fermion names exist and for correct (anti-)particles
      do i1 = 1,4
        cons = 0
        do i2 = 1,12
          anti = (-1)**(i1-1)  !particle(i1=1,3) or anti-particle(i1=2,4)
          if (fname(i1).eq.gname(anti*i2)) cons = 1
        enddo
        if (cons.eq.0) then
          write(nout,*)'Wrong particle name or wrong order of'
     &          ,' (anti-)particles'
          close(nout)
	  stop
        endif
      enddo
c check correct order of quarks and generations
      if ((quark(1).ne.quark(2)).or.(quark(3).ne.quark(4)).or.
     &     (gen(1).ne.gen(2)).or.(gen(3).ne.gen(4))) then
        write(nout,*)'Wrong order of particles or reaction not allowed'
        close(nout)
	stop
      endif
c check isospin
      if (((iso(1).eq.iso(2)).and.(iso(3).ne.iso(4))).or.
     &    ((iso(1).ne.iso(2)).and.(iso(3).eq.iso(4)))) then
        write(nout,*)'Reaction not allowed'
        close(nout)
	stop
      endif
      if ((iso(1).ne.iso(2)).and.(iso(1).ne.1.or.iso(2).ne.-1
     &     .or.iso(3).ne.-1.or.iso(4).ne.1)) then
        write(nout,*)'Wrong order of particles '
     &        ,'(CC order for mixed rections)'
        close(nout)
	stop
      endif
      if (iso(1).lt.iso(3)) then
        write(nout,*)'Wrong order of particles '
     &        ,'(Isospin: +1 +1 -1 -1)'
        close(nout)
	stop
      endif

      end

************************************************************************
*
*     fill common blocks with values from input_param/inputfile
*     fix derived parameters
*         call init_process and input_param and input before
*
************************************************************************
      subroutine fix_param(qwidthzero)

        use COLLIER

      implicit none

#ifdef mpiuse
      include "mpif.h"
#endif
      integer iNumProcs, iErr, iMyRank
      common/mpistuff/iNumProcs,iMyRank

      integer i,i1,i2,anti,temp_bos,temp_ferm
      integer qwidthzero
      real*8 wwidth,zwidth,dummy,ran1,ran_rcarry
      include 'COMMON/param.inc'
      include 'COMMON/param_bsm.inc'
      include 'COMMON/param_sesm.inc'
      include 'COMMON/param_thdm.inc'
      include 'COMMON/model.inc'
      include 'COMMON/processpar.inc'
      include 'COMMON/options.inc'
      include 'COMMON/xparam.inc'
      include 'COMMON/output.inc'
c couplings
      complex*16 cvf(-1:1,3,4),cvs(3)
      common/couplings/cvf,cvs
c boson masses
      real*8 mb(4),mb2(4),gb(4)
      common/bosmass/mb,mb2,gb
c particle names
      real*8 mass(-17:17),width(-17:17)
      character*8 ename(-7:7),gname(-17:17)
      common/particles/ename,gname,mass,width
c center of mass energy
      real*8 CMenergy2
      common/CMenergy2/CMenergy2
c IR
      real*8 lambda,lambda2
      common/ir/lambda,lambda2
c renormalization scale
      real*8 mudim2
      common/uv/mudim2
c fermion masses as flag (i*1d-20)
      complex*16 cme2,cmf2(5)
      real*8 me2,rme2,fmass_mf2(5),rmf2(5)
      common/fmass/me2,rme2,cme2,fmass_mf2,rmf2,cmf2
c fermion charges of isospin duplets
      real*8 qf(4)
      common/fcharge/qf
c factorization scale
      real*8 fsrfact,nfsrfact
      common/fsrfact/fsrfact,nfsrfact
c fitted constant for IBA
      real*8 chww,chzz
      common/IBA/chww,chzz
c random generator
      integer rangen,ranshift,randomseed
      common/random/rangen,ranshift,randomseed
c collier parameters
      real*8 deltauv,muv2,delta2ir,delta1ir,mir2,acc 
      integer*4 mode,nmax

c parameters for RGEs
      real*8 tb_1,ta_1,lam5_1,l12_1
      real*8 tb_2,ta_2,lam5_2,l12_2

c fixed quantities
      pi=4d0*datan(1d0)
c gauge boson width, 0=LO (used for Born preciction)
c   1=NLO (used for one-loop predictions, also for Born-part of one-loop)
      if (contrib.eq.3) then
         qwidth = 0
      else
         qwidth = 1
      endif
c conversion from input pdg W/Z masses to pole masses
      mz = mzi 
      mw = mwi 
      mz = mzi / dsqrt(1d0 + gzi**2/mzi**2)
      mw = mwi / dsqrt(1d0 + gwi**2/mwi**2)
      mh = mhi 
c abbreviations
      mw2 = mw**2
      mz2 = mz**2
      mh2 = mh**2
      do i=1,3
        ml2(i)=ml(i)**2
        mqp2(i)=mqp(i)**2
        mqm2(i)=mqm(i)**2
      enddo
c weak mixing angle
      cw = mw/mz
      cw2=cw**2
      sw2=1d0-cw2
      sw=dsqrt(sw2)
c alpha parametrization scheme
      if(qalp.eq.0)then
	alpha=alpha0
      elseif(qalp.eq.1)then
        alpha=dsqrt(2d0)*GF/pi*mw2*sw2
      elseif(qalp.eq.2)then
	alpha=alphaz
      endif
      el=dsqrt(4d0*pi*alpha)
c center of mass energy
      CMenergy2 = mh2 ! Higgs mass
c factorization scale
      fsrfact = (nfsrfact*mh)**2

C*** initialization of collier
	mode = 2	! 1=COLI, 2=DD, 3=COLI vs. DD
        nmax = 5
	Print *, "Prophecy4f relies on the following version of COLLIER:"
        call Init_cll(nmax)
        call SetMode_cll(mode)

        deltauv  = 10d0
        deltauv  = 0d0
        muv2     = CMenergy2 
        delta2ir = 0d0
        delta1ir = 0d0
        mir2     = 1d0
        call SetMuUV2_cll(muv2)
        call SetMuIR2_cll(mir2)
        call SetDeltaUV_cll(deltaUV)
        call SetDeltaIR_cll(delta1ir,delta2ir)

        acc  = 1d-6
        call SetReqAcc_cll(acc)

c old regularization parameters
        mudim2  = muv2
        lambda2 = mir2
        lambda  = sqrt(lambda2)

C*** end initialization of collier

c define particle properties
      do i1 = 1,12
        do i2 = 1,4
          anti = (-1)**(i2-1)               !f1/f3:fermion, f2/f4:anti-fermion
          if (fname(i2).eq.gname(anti*i1)) then
            gen(i2) = mod(i1-1,3)+1         !generation of fermion
            iso(i2) = mod((i1-1)/3,2)*2-1   !isospin of fermion
            quark(i2) = (i1-1)/6            !quark:1, no quark:0
          endif
        enddo
      enddo
c elmg. charges of fermions
      do i=1,4
         q(i) = (iso(i)-1d0)/2d0+quark(i)*2d0/3d0
      enddo
c color factor
      color = 3**(quark(1)+quark(3))
c type of process
      if (iso(1).gt.iso(2)) then
        cc = 1
        if (gen(1).eq.gen(3).and.quark(1).eq.quark(3)) then
          nc = 1
        else
          nc = 0
        endif
      else
        cc = 0
        nc = 1
      endif
c calculate renormalization constants (real)
      call calcrenc
      call calcrenc_ferm
      call calcrenc_bgf

c vector boson width
      temp_bos = qbos
      temp_ferm = qferm
      qbos = 1
      qferm = 1
      if (qwidth.eq.0) then
         gw = wwidth(0,0)       !0:LO width
         gz = zwidth(0,0)
      else
         gw = wwidth(1,1)       !1:NLO width (including QCD corrections)
         gz = zwidth(1,1)
      endif
      if (qwidthzero.eq.1) then 
         gw = 0d0
         gz = 0d0
      endif
      qbos = temp_bos
      qferm = temp_ferm
      gt = GF*(mqp2(3)-mw2)**2*(mqp2(3)+2d0*mw2)
     &         /(8d0*pi*dsqrt(2d0)*mqp(3)**3)

c complex parameters
      xmz2 = dcmplx(mz2,-mz*gz)
      xmz = cdsqrt(xmz2)
      xmw2 = dcmplx(mw2,-mw*gw)
      xmw = cdsqrt(xmw2)
      xmh2 = dcmplx(mh2,0d0)
      xmh = cdsqrt(xmh2)
      do i=1,3
        xml2(i)  = ml2(i)
        xmqm2(i) = mqm2(i)
        xmqp2(i) = mqp2(i)
      enddo
      xmqp2(3) = dcmplx(mqp2(3),-mqp(3)*gt)
      null = dcmplx(0d0,0d0)
      xcw = cdsqrt(dcmplx(mw2,-mw*gw)
     &           /dcmplx(mz2,-mz*gz) )
      xcw2=xcw**2
      xsw2=1d0-xcw2
      xsw=cdsqrt(xsw2)
c boson masses and widths
      mb(1) = 0d0
      mb2(1) = 0d0
      gb(1) = 0d0
      mb(2) = mz
      mb2(2) = mz2
      gb(2) = gz
      mb(3) = mw
      mb2(3) = mw2
      gb(3) = gw
      mb(4) = mh
      mb2(4) = mh2
      gb(4) = 0d0

c external fermion masses and masses for loops connected to external fermions
      do i=1,4
        if (quark(i).eq.0) then
          if (iso(i).eq.-1) then
            mf(i) = ml(gen(i))
            mf2(i) = ml2(gen(i))
          else
            mf(i) = 0d0
            mf2(i) = 0d0
          endif
        else
          if (iso(i).eq.-1) then
            mf(i) = mqm(gen(i))
            mf2(i) = mqm2(gen(i))
          else
            mf(i) = mqp(gen(i))
            mf2(i) = mqp2(gen(i))
          endif
        endif
      enddo

c external particle names defined for subroutines check_topology
      ename(1)='H0'
      ename(2)='XXX'
      do i = 1,4
        ename(i+2) = fname(i)
      enddo
      ename(7) = 'ga'  ! photon
      do i=3,6
        if (ename(i)(1:4).eq.'anti') then
          ename(-i) = ename(i)(6:8)
        else
          ename(-i) = 'anti-'//ename(i)
        endif
      enddo
      ename(-7) = ename(7)

c electron mass as flag
      mxe = 1d-15
      mxe2 = mxe**2
c external fermion masses as flag (i*1d-20)
c convention:(f1,f2)/(f3,f4) form isospin duplet
c ->fmass_mf(1)/fmass_mf(4):up-type, fmass_mf(2)/fmass_mf(3):down-type
      me2  = ml2(1)
      rme2 = me2
      cme2 = me2
      fmass_mf2(5) = me2
      if (quark(1).eq.0) then
        fmass_mf2(1) = 0d0
        fmass_mf2(2) = ml2(gen(2))
      else
        fmass_mf2(1) = mqp2(gen(1))
        fmass_mf2(2) = mqm2(gen(2))
      endif
      if (quark(3).eq.0) then
        fmass_mf2(3) = ml2(gen(3))
        fmass_mf2(4) = 0d0
      else
        fmass_mf2(3) = mqm2(gen(3))
        fmass_mf2(4) = mqp2(gen(4))
      endif
      do i = 1,5
        rmf2(i) = i*1d-20
        cmf2(i) = rmf2(i)
	fmass_mf2(i) = fmass_mf2(i) * (1d0+1d-14) ! to distinguish them from internal fermions
      enddo
c fermion charges of isospin duplets
      qf(1) = quark(1)*2d0/3d0
      qf(2) = -1d0 + quark(2)*2d0/3d0
      qf(3) = -1d0 + quark(3)*2d0/3d0
      qf(4) = quark(4)*2d0/3d0
c couplings of fermions to gauge bosons
      do i=1,4
        cvf(-1,1,i) = -el*q(i)
        if (qcomp.eq.0) then
          cvf(+1,1,i) = -el*q(i)
          cvf(-1,2,i) = -el*(q(i)*sw/cw-iso(i)/2d0/sw/cw)
          cvf(+1,2,i) = -el* q(i)*sw/cw
          cvf(-1,3,i) =  el/dsqrt(2d0)/sw
        elseif (qcomp.eq.1) then
          cvf(+1,1,i) = -el*q(i)
          cvf(-1,2,i) = -el*(q(i)*xsw/xcw-iso(i)/2d0/xsw/xcw)
          cvf(+1,2,i) = -el* q(i)*xsw/xcw
          cvf(-1,3,i) =  el/dsqrt(2d0)/xsw
        endif
        cvf(+1,3,i) = 0d0
      enddo
c coupling of higgs to vector bosons
      if (qcomp.eq.0) then
        cvs(3)=el*mw/sw        ! coupling Higgs-W-W
        cvs(2)=el*mw/sw/cw2    ! coupling Higgs-Z-Z
      elseif (qcomp.eq.1)then
        cvs(3)=el*xmw/xsw          ! coupling Higgs-W-W
        cvs(2)=el*xmw/xsw/xcw2      ! coupling Higgs-Z-Z
      endif

c set BSM parameters
	if (model.eq.1) then
	  call bsm_setparams()
	  ta_1   = ta_in
	  l12_1  = l12_in
cxxx
c	  write(*,*) '111 ',ta_1,l12_1
c	  write(*,*) 'sa, l12',ta_1/sqrt(1d0+ta_1**2),l12_1
	  call RGE_sesm(ta_1,l12_1,mrenbsm1**2,
     &	                ta_2,l12_2,mrenbsm2**2)
	  mren2bsm = mrenbsm2**2
	  ta   = ta_2
	  l12  = l12_2
	  sa   = ta/sqrt(1d0+ta**2)
	  ca   = sa/ta
cxxx
c	  write(*,*) '222 ',ta_2,l12_2
c	  write(*,*) 'sa, l12',sa,l12
c	stop
	  call bsm_setparams()
	  call sesm_setcoupl(hboson)
          cvs(2) = cvs(2)*CVH
          cvs(3) = cvs(3)*CVH
          if (hboson.eq.1) then
            CMenergy2 = mh02
          elseif (hboson.eq.2) then
            CMenergy2 = mHH2
          endif
	  mren2bsm = mrenbsm2**2

	elseif (model.eq.2) then
	  call bsm_setparams()
	  tb_1   = tb_in
	  ta_1   = ta_in
	  lam5_1 = lam5_in
cxxx
c	  write(*,*) '111 ',tb_1,ta_1,lam5_1
c	  write(*,*) 'tb, sa, lam5',tb_1,ta_1/sqrt(1d0+ta_1**2),lam5_1
	  call RGE_thdm(tb_1,ta_1,lam5_1,mrenbsm1**2,
     &	                tb_2,ta_2,lam5_2,mrenbsm2**2)
	  mren2bsm = mrenbsm2**2
	  tb   = tb_2
	  ta   = ta_2
	  lam5 = lam5_2
	  sb   = tb/sqrt(1d0+tb**2)
	  cb   = sb/tb
	  sa   = ta/sqrt(1d0+ta**2)
	  ca   = sa/ta
cxxx
c	  write(*,*) '222 ',tb_2,ta_2,lam5_2
c	  write(*,*) 'cba, sa, tb, lam5',cb*ca+sb*sa,sa,tb,lam5
c	stop
	  call bsm_setparams()
	  call thdm_setcoupl(modeltype,hboson)
          cvs(2) = cvs(2)*CVH
          cvs(3) = cvs(3)*CVH
          if (hboson.eq.1) then
            CMenergy2 = mh02
          elseif (hboson.eq.2) then
            CMenergy2 = mHH2
          endif
	endif
        if (model.ne.0) then
           mh2 = CMenergy2
           mh  = sqrt(mh2)
        endif

c particle masses and widths for phase-space generator
      do i=-16,16
        mass(i)=0d0
        width(i)=0d0
      enddo
      mass(-13) = mw
      width(-13)= gw
      mass(-14) = mz
      width(-14)= gz
      do i=1,16
        mass(i)=mass(-i)
        width(i)=width(-i)
      enddo

C*** small masses in collier

	call clearminf2_cll
	do i=1,5
	  if (fmass_mf2(i).ne.0d0) 
     &	    call addminf2_cll(dcmplx(fmass_mf2(i)))
	enddo

c calculate renormalization constants (complex)
      call xcalcrenc
      call xcalcrenc_ferm
      call xcalcrenc_bgf
      if (model.eq.1) then
        call xcalcrenc_sesm
        call xcalcrenc_ferm_sesm
	call transinput_sesm(0)
	call transinput_sesm(1)
      elseif (model.eq.2) then
        call xcalcrenc_thdm
        call xcalcrenc_ferm_thdm
	call transinput_thdm(0)
	call transinput_thdm(1)
      endif

c fitted constant for IBA
      chww = 0.04
      chzz = 0.03
c shift of random numbers for uncorrelated results
      do i = 1,ranshift
        if (rangen.eq.1) then
          dummy = ran1()
        elseif (rangen.eq.2) then
          dummy = ran_rcarry()
        elseif (rangen.eq.3) then
          call ranlux(dummy,1)
        else
	  Write(nout,*) "rangen ",rangen," not supported"
	  Stop
	endif
      enddo

      end

**************************************************************
*
*     initialize separation cuts and photon recombination
*
**************************************************************
      subroutine init_cuts
      implicit none
      integer i1,i2
      include 'COMMON/param.inc'
      include 'COMMON/processpar.inc'
c inputcuts
      real*8 ecutp,ecutl,ecutq,scutqq,ccutpl,ccutpq,ccutll,ccutqq,ccutlq
      common/inputcuts/ecutp,ecutl,ecutq,scutqq,ccutpl,ccutpq,ccutll
     &     ,ccutqq,ccutlq
c cuts
      real*8 ecut(5),scut(5,5),ccut(5,5),precomb(3)
      integer qcuts
      common/cuts/ecut,scut,ccut,precomb,qcuts

c cuts are probably wrong, don't use them.
c the problem:
c convolution contributions need born, but possible that born cut away but not
c convolution kinematics. this happens with no recombination and forcing Z almost
c on-shell: Born m_ee above mz and cut away, but collinear radiation leads to accepted
c event when not recombining. then using wrong/invalid born.
c this occurs in weight_bornvirt.
      qcuts = 0   ! no separation cuts
      if (qcuts.eq.0)then
        ecutp = 0d0
        ecutl = 0d0
        ecutq = 0d0
        scutqq = 0d0
        ccutpl = 0d0
        ccutpq = 0d0
        ccutll = 0d0
        ccutqq = 0d0
        ccutlq = 0d0
c photon cuts for 1->5 Born cross section
      elseif (qcuts.eq.1) then
        ecutp=10d0             ! photon energy cut
        ecutl=0d0              ! charged-lepton energy cut
        ecutq=0d0              ! quark energy cut
        scutqq=0d0             ! quark-quark invariant mass cut
        ccutpl=5d0             ! angular cut between photon and charged lepton
        ccutpq=5d0             ! angular cut between photon and quark
        ccutll=0d0             ! angular cut between charged leptons
        ccutqq=0d0             ! angular cut between quarks
        ccutlq=0d0             ! angular cut between charged lepton and quark
c cuts for Born cross section (1->4, or 1->5)
      elseif (qcuts.eq.2) then
        ecutp=10d0             ! photon energy cut
        ecutl=10d0             ! charged-lepton energy cut
        ecutq=10d0             ! quark energy cut
        scutqq=10d0            ! quark-quark invariant mass cut
        ccutpl=5d0             ! angular cut between photon and charged lepton
        ccutpq=5d0             ! angular cut between photon and quark
        ccutll=5d0             ! angular cut between charged leptons
        ccutqq=0d0             ! angular cut between quarks
        ccutlq=5d0             ! angular cut between charged lepton and quark
c cuts for radiative corrections
      elseif(qcuts.eq.3)then
        ecutp=0d0              ! photon energy cut
        ecutl=50d0             ! charged-lepton energy cut
        ecutq=50d0             ! quark energy cut
        scutqq=50d0            ! quark-quark invariant mass cut
        ccutpl=0d0             ! angular cut between photon and charged lepton
        ccutpq=0d0             ! angular cut between photon and quark
        ccutll=55d0             ! angular cut between charged leptons
        ccutqq=0d0             ! angular cut between quarks
        ccutlq=55d0             ! angular cut between charged lepton and quark
      endif
c initialize arrays for cuts
      do i1=1,5
        ecut(i1)=0d0
        do i2=1,5
          scut(i1,i2)=0d0
          ccut(i1,i2)=0d0
        enddo
      enddo
c filling cut parameters, note that processpar:1..4, cuts:1..7
      if (qcuts.ne.0) then
        ecut(5)=ecutp
        do i1=1,4
          if (quark(i1).eq.0.and.iso(i1).lt.0) then     ! charged leptons
            ecut(i1)=ecutl
            ccut(i1,5)=ccutpl
          elseif (quark(i1).eq.1) then                  ! quarks
            ecut(i1)=ecutq
            ccut(i1,5)=ccutpq
          endif
        do i2=i1+1,4
          if (quark(i1).eq.1.and.quark(i2).eq.1) then
            scut(i1,i2)=scutqq
            ccut(i1,i2)=ccutqq
          elseif (quark(i1).eq.0.and.iso(i1).lt.0.and.
     &           quark(i2).eq.0.and.iso(i2).lt.0) then
            ccut(i1,i2)=ccutll
          elseif ((quark(i1).eq.0.and.iso(i1).lt.0.and.quark(i2)
     &        .eq.1).or.(quark(i1).eq.1.and.quark(i2).eq.0.and.
     &        iso(i2).lt.0)) then
            ccut(i1,i2)=ccutlq
          endif
        enddo
        enddo
      endif

c calculating cut parameters
c angle -> cos(angle)
      do i1=1,5
      do i2=1,5
        ccut(i1,i2)=dcos(ccut(i1,i2)*pi/180d0)
      enddo
      enddo
c mass -> mass squared
      do i1=1,5
      do i2=1,5
        scut(i1,i2)=scut(i1,i2)**2
      enddo
      enddo
c filling whole common block
      do i1=1,5
      do i2=i1+1,5
        ccut(i1,i2)=min(ccut(i1,i2),ccut(i2,i1))
        scut(i1,i2)=max(scut(i1,i2),scut(i2,i1))
      enddo
      enddo

      end

**************************************************************
*
*     initialize fast evaluation of tensor integrals
*       (calculate virtual corrections for 10 ph-sp points)
**************************************************************
      subroutine init_fast
      implicit none
      include 'COMMON/options.inc'
      include 'COMMON/max.inc'
      include 'COMMON/output.inc'
      integer i,j,nocut,disc,ngen,nparticles
      real*8 random(18),inmom(0:3),outmom(0:3,5),distmom(0:3,5)
      real*8 dens(maxchan),denstotal,sij(7,7)
      real*8 dM2_virt_EW,dM2_virt_QCD,dummy,ran1,ran_rcarry
c center of mass energy
      real*8 CMenergy2
      common/CMenergy2/CMenergy2
c random generator
      integer rangen,ranshift,randomseed
      common/random/rangen,ranshift,randomseed

c	write(*,*) 'Irrelevant when using collier !'
c	stop

      if (qfast.eq.0) then
c        call fastoff	! irrelevant for collier
        do j = 1,12 ! same random numbers independent of qfast
          do i = 1,18
            if (rangen.eq.1) then
              random(i) = ran1()
            elseif (rangen.eq.2) then
              random(i) = ran_rcarry()
            elseif (rangen.eq.3) then
              call ranlux(random(i),1)
            else
	      Write(nout,*) "rangen ",rangen," not supported"
	      Stop
            endif
          enddo
        enddo
      else
        ngen = 1
        nparticles = 4
        do j = 1,12 !2 more, in case of discarded event
          do i = 1,18
            if (rangen.eq.1) then
              random(i) = ran1()
            elseif (rangen.eq.2) then
              random(i) = ran_rcarry()
            elseif (rangen.eq.3) then
              call ranlux(random(i),1)
            else
	      Write(nout,*) "rangen ",rangen," not supported"
	      Stop
            endif
          enddo
c generate phase-space point
          inmom(0) = dsqrt(CMenergy2) ! Higgs mass
          do i = 1,3
            inmom(i) = 0d0
          enddo
          call phasespace(random,ngen,inmom,outmom,distmom
     &         ,sij,dens,denstotal,nparticles,nocut,disc)
          call setprods(outmom(0,1),nparticles)
          call calcsme()
c          call fastinit(7)	! irrelevent for collier
          dummy = dM2_virt_QCD() !QCD before EW, keep same order!
          dummy = dM2_virt_EW()
        enddo
      endif

      end

**************************************************************
*
*     begin ouput, number of error messages, input parameters,...
*
**************************************************************
      subroutine begin_output(nout,maxout,moreout)
      implicit none
C      include 'COMMON/output.inc'
      include 'COMMON/param.inc'
      include 'COMMON/param_bsm.inc'
      include 'COMMON/param_sesm.inc'
      include 'COMMON/param_thdm.inc'
      include 'COMMON/model.inc'
      include 'COMMON/processpar.inc'
      include 'COMMON/options.inc'
      include 'COMMON/techpar.inc'
      integer i1,i2,length
      integer nout,maxout,moreout
      real*8 narrwidappr,nwa,nwaborn,nwacorr,br
      real*8 width_wff,width_zff,width_hww,width_hzz,wwidth,zwidth
      real*8 Gh0WW_lo,Gh0WW_nlo,GHHWW_lo,GHHWW_nlo
      real*8 Gh0ZZ_lo,Gh0ZZ_nlo,GHHZZ_lo,GHHZZ_nlo
      integer values(8),i
      character date*8,time*10,zone*5
c cuts
      real*8 ecut(5),scut(5,5),ccut(5,5),precomb(3)
      integer qcuts
      common/cuts/ecut,scut,ccut,precomb,qcuts
c slicing cuts
      real*8 deltaE,deltaTh
      common/slicuts/deltaE,deltaTh
c random generator
      integer rangen,ranshift,randomseed
      common/random/rangen,ranshift,randomseed
c IR
      real*8 lambda,lambda2
      common/ir/lambda,lambda2
c renormalization scale
      real*8 mudim2
      common/uv/mudim2
c outputfile
      character*90 outputfile
      common/outputfile/outputfile
c smearing
      integer smear
      real*8 smrat
      common/smearing/smrat,smear
c fitted constant for IBA
      real*8 chww,chzz
      common/IBA/chww,chzz

      character*30 str_model(0:4),str_hboson(0:2)
      character*35 str_rscheme_sesm(0:7),str_rscheme_thdm(0:8)

      str_model(0) = 'Standard Model'
      str_model(1) = 'Singlet Extension of the SM'
      str_model(2) = 'Two-Higgs Doublet Model'
      str_model(4) = 'SM with 4th fermion generation'
      if (model.eq.1) then
        str_hboson(1) = 'light Higgs boson h0'
        str_hboson(2) = 'heavy Higgs boson HH '
      elseif (model.eq.2) then
        str_hboson(1) = 'light CP-even Higgs boson h0'
        str_hboson(2) = 'heavy CP-even Higgs boson HH '
      endif

      str_rscheme_sesm(0) = 'alpha MSbar (running la12)'
      str_rscheme_sesm(1) = 'alpha MSbar a la FJ (running la12)'
      str_rscheme_sesm(2) = 'on-shell scheme (running la12)'
      str_rscheme_sesm(3) = 'alpha MSbar (running la1)'
      str_rscheme_sesm(4) = 'alpha MSbar a la FJ (running la1)'
      str_rscheme_sesm(5) = 'on-shell scheme (running la1)'
      str_rscheme_sesm(6) = 'BFM-inspired scheme 1'
      str_rscheme_sesm(7) = 'BFM-inspired scheme 2'

      str_rscheme_thdm(0) = 'alpha/beta MSbar'
      str_rscheme_thdm(1) = 'alpha/beta MSbar a la FJ'
      str_rscheme_thdm(2) = 'lambda3/beta MSbar'
      str_rscheme_thdm(3) = 'lambda3/beta MSbar a la FJ'
      str_rscheme_thdm(4) = 'alpha/beta MSbar on-shell (nu2)'
      str_rscheme_thdm(5) = 'alpha/beta MSbar on-shell (nu1,nu2)'
      str_rscheme_thdm(6) = 'alpha/beta MSbar on-shell (nu1)'
      str_rscheme_thdm(7) = 'BFM-inspired scheme 1'
      str_rscheme_thdm(8) = 'BFM-inspired scheme 2'


      write(nout,*)''
      write(nout,*)' Model and process:'
      write(nout,*)' ------------------'
      write(nout,*)''
      write(nout,*) '   model: ',str_model(model)
      if (model.eq.0 .or. model .eq. 4) then
        write(nout,*) '   initial state:   SM Higgs boson H'
      elseif (model.eq.1) then
        write(nout,*) '   initial state: ',str_hboson(hboson)
      elseif (model.eq.2) then
        write(nout,'(a16,i4)') '   model type: ',modeltype
        write(nout,*) '   initial state: ',str_hboson(hboson)
      endif

      write(nout,*)'   -> final state:  '
     &    ,fname(1)(1:length(fname(1))),' ',fname(2)(1:length(fname(2)))
     &    ,' ',fname(3)(1:length(fname(3))),' '
     &    ,fname(4)(1:length(fname(4))),' (+Photon/Gluon)'
      write(nout,*)''
      write(nout,*)' Input parameters:'
      write(nout,*)' -----------------'
C      write(nout,'(a19,f12.8,a20,f8.4)')'    1/alpha(0) = 1/',1d0/alpha0
C     &     ,'    alphas = ',alphas
      write(nout,'(a16,f8.4)') '        alphas =',alphas
      write(nout,'(a17,g14.7,a7)') 'GF = ',gf,' GeV^-2'
      write(nout,'(a17,f9.5,a4,a10,f9.5,a4,a10,f9.4,a4)')
     &      'MW = ',mwi,' GeV','     MZ = ',mzi,' GeV'
     &     ,'     MH = ',mhi,' GeV'
      write(nout,'(a17,f9.5,a4,a10,f9.5,a4)')
     &      'GW = ',gwi,' GeV','     GZ = ',gzi,' GeV'
      write(nout,'(a17,f9.5,a4,a10,f9.5,a4,a10,f9.5,a4)')
     &      'me = ',ml(1)*1d3,' MeV','    mmu = ',ml(2),' GeV'
     &     ,'   mtau = ',ml(3),' GeV'
      write(nout,'(a17,f9.5,a4,a10,f9.5,a4,a10,f9.5,a4)')
     &      'mu = ',mqp(1),' GeV','     mc = ',mqp(2),' GeV'
     &     ,'     mt = ',mqp(3),' GeV'
      write(nout,'(a17,f9.5,a4,a10,f9.5,a4,a10,f9.5,a4)')
     &      'md = ',mqm(1),' GeV','     ms = ',mqm(2),' GeV'
     &     ,'     mb = ',mqm(3),' GeV'
      write(nout,*)
      if (qsm4 .ne. 0) then
         write(nout,'(a17,f9.5,a4,a10,f9.5,a4)')
     &         'mu4 = ',mu4,' GeV','mn4 = ',mn4,' GeV'
         write(nout,'(a17,f9.5,a4,a10,f9.5,a4)')
     &         'md4 = ',md4,' GeV','ml4 = ',ml4,' GeV'
         write(nout,*)
      endif

      if (model.eq.1) then
        write(nout,*) '   SESM parameters:'
        write(nout,'(a17,f9.5,a4,a10,f9.5,a4)')
     &      'Mh0 = ',Mh0,' GeV','    MHH = ',MHH,' GeV'

        write(nout,'(a50,f9.5,a4)')
     &     'Input renormalization scale:   mrenbsm1 = ',mrenbsm1,' GeV'
        if (ta_in.eq.1d30) then
	  ca_in = 0d0
	  sa_in = 1d0
	else
	  ca_in = 1d0/sqrt(1d0+ta_in**2)
	  sa_in = ta_in*ca_in
	endif
      write(nout,'(a17,f9.5,a4,a10,f9.5,a4,a10,f9.5)') 'ta = ',ta_in,
     &     '    ','     ca = ',ca_in,'    ','     sa = ',sa_in
      write(nout,'(a17,f9.5)') ' l12 = ',l12_in

      write(nout,'(a50,f9.5,a4)')
     &     'Output renormalization scale:   mrenbsm2 = ',mrenbsm2,' GeV'
      write(nout,'(a17,f9.5,a4,a10,f9.5,a4,a10,f9.5)')
     &      'ta = ',ta,'    ','     ca = ',ca,'    ','     sa = ',sa
      write(nout,'(a17,f9.5)') ' l12 = ',l12 
      write(nout,'(a17,i2,a3,a35)')
     &	'Ren. scheme ',renscheme,':  ',str_rscheme_sesm(renscheme)

      write(nout,'(a50)') 'Linearized conversion to different ren. schemes:'
      do i=0,7
      write(nout,'(a9,i2,a10,g18.10,a4,a7,g18.10)')
     &     'RS = ',i,'  sa = ',sa_RSsesm_lin(i),'    ',
     &     'l12  = ',l12_RSsesm_lin(i)
      enddo
      write(nout,'(a42)') 'Bare matching to different ren. schemes:'
      do i=0,7
      if (conv_sesm(i).eq.0) then
        write(nout,'(a9,i2,a10,g18.10,a4,a7,g18.10)') 'RS = ',i,
     &     '  sa = ',sa_RSsesm(i),'    ','l12  = ',l12_RSsesm(i)
      else
        write(nout,'(a9,i2,a10,g18.10,a4,a7,g18.10,a22)') 'RS = ',i,
     &     '  sa = ',sa_RSsesm(i),'    ','l12  = ',l12_RSsesm(i),
     &     '  convergence problem!'
      endif
      enddo
      write(nout,*)

      if ((sa_in.eq.0d0).or.(abs(sa_in).eq.1d0)) then
        if (l12_in.eq.0d0) then
          write(nout,*) 
     &       'sa=0 or +-1, lambda_12=0 corresponds to the SM.'
          write(nout,*) 'Program stopped - switch to SM mode!'
          write(nout,*) 
        else
          write(nout,*) 
     &	     'Scenario ill-defined for sa=0 or +-1, lambda_12=/=0,'
          write(nout,*) 'because lambda_1 -> infinity.'
          write(nout,*) 'Program stopped - change input!'
          write(nout,*) 
        endif
	close(nout)
        stop
      endif

      if (sign(1d0,l12_in).ne.sign(1d0,sa_in)) then
        write(nout,*) 'l12 and sa must have same sign in SESM !'
        write(nout,*) 'Program stopped - change input!'
        write(nout,*) 
	close(nout)
        stop
      endif

      write(nout,*)' Scalar 4-point self-couplings:'
      write(nout,*)' ------------------------------'
      write(nout,'(a17,g16.8)') 'lambda_1  = ',lam1_sesm
      write(nout,'(a17,g16.8)') 'lambda_2  = ',lam2_sesm
      write(nout,'(a17,g16.8)') 'lambda_12 = ',lam12_sesm
      if ((.not.(abs(lam1_sesm) .lt.lam_np_sesm)).or.
     &	  (.not.(abs(lam2_sesm) .lt.lam_np_sesm)).or.
     &	  (.not.(abs(lam12_sesm).lt.lam_np_sesm))) then
        write(nout,'(a42,g16.8)')
     &		'   WARNING: lambda > lambda_nonpert_sesm = ',
     &		lam_np_sesm
        write(nout,*)'     => Scenario might be non-perturbative!'
      endif
      write(nout,*)''

cxxx test output for Higgs decays into on-shell W/Z bosons
c	call hvv_sesm(Gh0WW_lo,Gh0WW_nlo,GHHWW_lo,GHHWW_nlo,
c     &                Gh0ZZ_lo,Gh0ZZ_nlo,GHHZZ_lo,GHHZZ_nlo)
c        write(nout,'(a22,3x,a2,24x,a3)') 'h0/HH -> VV(on-shell):','LO','NLO'
c        write(nout,*) 'Gamma_h0->WW[GeV]:   ',Gh0WW_lo,Gh0WW_nlo
c        write(nout,*) 'Gamma_HH->WW[GeV]:   ',GHHWW_lo,GHHWW_nlo
c        write(nout,*) 'Gamma_h0->ZZ[GeV]:   ',Gh0ZZ_lo,Gh0ZZ_nlo
c        write(nout,*) 'Gamma_HH->ZZ[GeV]:   ',GHHZZ_lo,GHHZZ_nlo
c	stop
cxxx

      elseif (model.eq.2) then
        write(nout,*) '   THDM parameters:'
        write(nout,'(a17,f9.5,a4,a10,f9.5,a4)')
     &      'Mh0 = ',Mh0,' GeV','    MHH = ',MHH,' GeV'
        write(nout,'(a17,f9.5,a4,a10,f9.5,a4)')
     &      'MA0 = ',MA0,' GeV','    MHp = ',MHp,' GeV'

        write(nout,'(a50,f9.5,a4)')
     &     'Input renormalization scale:   mrenbsm1 = ',mrenbsm1,' GeV'
	cb_in = 1d0/sqrt(1d0+tb_in**2)
	sb_in = tb_in*cb_in
	ca_in = 1d0/sqrt(1d0+ta_in**2)
	sa_in = ta_in*ca_in
      write(nout,'(a17,f9.5,a4,a10,f9.5,a4,a10,f9.5)') 'tb = ',tb_in,
     &     '    ','     cb = ',cb_in,'    ','     sb = ',sb_in
      write(nout,'(a17,f9.5,a4,a10,f9.5,a4,a10,f9.5)') 'ta = ',ta_in,
     &     '    ','     ca = ',ca_in,'    ','     sa = ',sa_in
      write(nout,'(a40,f9.5,a4,a10,f9.5)')
     &      'ba = beta-alpha       cba = ',ca_in*cb_in+sa_in*sb_in,
     &      '    ','sba = ',sb_in*ca_in-cb_in*sa_in
      write(nout,'(a17,f9.5)') 'lam5 = ',lam5_in

      write(nout,'(a50,f9.5,a4)')
     &     'Output renormalization scale:   mrenbsm2 = ',mrenbsm2,' GeV'
      write(nout,'(a17,f9.5,a4,a10,f9.5,a4,a10,f9.5)')
     &      'tb = ',tb,'    ','     cb = ',cb,'    ','     sb = ',sb
      write(nout,'(a17,f9.5,a4,a10,f9.5,a4,a10,f9.5)')
     &      'ta = ',ta,'    ','     ca = ',ca,'    ','     sa = ',sa
      write(nout,'(a40,f9.5,a4,a10,f9.5)')
     &      'ba = beta-alpha       cba = ',cba,'    ','sba = ',sba
      write(nout,'(a17,f9.5)') 'lam5 = ',lam5
      write(nout,'(a17,i2,a3,a35)')
     &	'Ren. scheme ',renscheme,':  ',str_rscheme_thdm(renscheme)

      write(nout,'(a50)') 'Linearized conversion to different ren. schemes:'
      do i=0,8
      write(nout,'(a9,i2,a10,g18.10,a9,g18.10,a4,a7,g18.10)')
     &     'RS = ',i,'tb = ',tb_RSthdm_lin(i),
     &	   '  sa = ',sa_RSthdm_lin(i),'    ',
     &     'lam5 = ',lam5_RSthdm_lin(i)
      enddo
      write(nout,'(a42)') 'Bare matching to different ren. schemes:'
      do i=0,8
      if (conv_thdm(i).eq.0) then
        write(nout,'(a9,i2,a10,g18.10,a9,g18.10,a4,a7,g18.10)')
     &     'RS = ',i,'tb = ',tb_RSthdm(i),'  sa = ',sa_RSthdm(i),'    ',
     &       'lam5 = ',lam5_RSthdm(i)
      else
        write(nout,'(a9,i2,a10,g18.10,a9,g18.10,a4,a7,g18.10,a22)')
     &     'RS = ',i,'tb = ',tb_RSthdm(i),'  sa = ',sa_RSthdm(i),'    ',
     &       'lam5 = ',lam5_RSthdm(i),
     &     '  convergence problem!'
      endif
      enddo

      write(nout,*)
      write(nout,*)' Scalar 4-point self-couplings:'
      write(nout,*)' ------------------------------'
      write(nout,'(a17,g16.8)') 'lambda_1 = ',lam1_thdm
      write(nout,'(a17,g16.8)') 'lambda_2 = ',lam2_thdm
      write(nout,'(a17,g16.8)') 'lambda_3 = ',lam3_thdm
      write(nout,'(a17,g16.8)') 'lambda_4 = ',lam4_thdm
      write(nout,'(a17,g16.8)') 'lambda_5 = ',lam5_thdm
      if ((.not.(abs(lam1_thdm).lt.lam_np_thdm)).or.
     &	  (.not.(abs(lam2_thdm).lt.lam_np_thdm)).or.
     &	  (.not.(abs(lam3_thdm).lt.lam_np_thdm)).or.
     &	  (.not.(abs(lam4_thdm).lt.lam_np_thdm)).or.
     &	  (.not.(abs(lam5_thdm).lt.lam_np_thdm))) then
        write(nout,'(a42,g16.8)')
     &		'   WARNING: lambda > lambda_nonpert_thdm = ',
     &		lam_np_thdm
        write(nout,*)'     => Scenario might be non-perturbative!'
      endif
      write(nout,*)''

      endif

c derived parameters
      write(nout,*)' Derived parameters:'
      write(nout,*)' -------------------'
      write(nout,'(a19,f12.8)')'    1/alpha = 1/',1d0/alpha
      If(model.EQ.1 .OR. model.EQ.2) Then
        write(nout,'(a56,g10.3,a5)') '   W/Z pole masses and widths (SM calculation with MH = ',mh,' GeV)'
      Else
        write(nout,*) '   W/Z pole masses and widths'
      Endif
      write(nout,'(a17,f9.5,a4,a10,f9.5,a4)')
     &      'MW = ',mw,' GeV','     MZ = ',mz,' GeV'
      write(nout,'(a17,f9.5,a4,a10,f9.5,a4,a10,f9.5,a4,a10,f9.5,a4)')
     &     'GW = ',gw,' GeV','     GZ = ',gz,' GeV',
     &     '     gt = ',gt,' GeV'
      write(nout,*)
c options
      write(nout,*)' Options:'
      write(nout,*)' --------'
      write(nout,*)
      write(nout,'(a33,i9)') '   number of weighted events   = ',nevents
      write(nout,'(a33,i9)') '   number of unweighted events = ',
     &     nunwevents
CAM removed since it is no longer available in README
C      write(nout,'(a16,i2,(a))')  'lheoutput =', lheoutput
      write(nout,*)
      if(contrib.eq.1)then
         write(nout,'(a16,i2,(a))') 'contrib =',contrib
     &        ,' : Born cross section + radiative corrections'
      elseif(contrib.eq.2)then
        write(nout,'(a16,i2,(a))') 'contrib =',contrib
     &        ,' : Improved Born approximation'
      elseif(contrib.eq.3)then
        write(nout,'(a16,i2,(a))') 'contrib =',contrib
     &        ,' : Born cross section'
      elseif(contrib.eq.4)then
        write(nout,'(a16,i2,(a))') 'contrib =',contrib
     &        ,' : Bremsstrahlungs process H->4f+Photon'
      elseif(contrib.eq.5)then
        write(nout,'(a16,i2,(a))') 'contrib =',contrib
     &        ,' : slicing endpoint only'
      elseif(contrib.eq.6)then
        write(nout,'(a16,i2,(a))') 'contrib =',contrib
     &        ,' : virtual + subtraction endpoint only'
      elseif(contrib.eq.7)then
        write(nout,'(a16,i2,(a))') 'contrib =',contrib
     &        ,' : FSR only'
      endif
      if (qqcd.eq.0) then
         write(nout,'(a16,i2,(a))') 'qqcd =',qqcd
     &        ,' : ONLY EW corrections'
      elseif (qqcd.eq.1) then
         write(nout,'(a16,i2,(a))') 'qqcd =',qqcd
     &        ,' : EW+QCD corrections'
      elseif (qqcd.eq.2) then
         write(nout,'(a16,i2,(a))') 'qqcd =',qqcd
     &        ,' : ONLY QCD corrections'
      elseif (qqcd.eq.3) then
         write(nout,'(a16,i2,(a))') 'qqcd =',qqcd
     &        ,' : ONLY QCD corrections, category a'
      elseif (qqcd.eq.4) then
         write(nout,'(a16,i2,(a))') 'qqcd =',qqcd
     &        ,' : ONLY QCD corrections, category b'
      elseif (qqcd.eq.5) then
         write(nout,'(a16,i2,(a))') 'qqcd =',qqcd
     &        ,' : ONLY QCD corrections, category c'
      elseif (qqcd.eq.6) then
         write(nout,'(a16,i2,(a))') 'qqcd =',qqcd
     &        ,' : ONLY QCD corrections, category d'
      endif
      if (model.eq.4 .and. qsm4 .eq. 0) then
         write(nout,'(a16, i2,(a))')  'qsm4 =', qsm4,
     &        ' : SM'
      elseif (model.eq.4 .and. qsm4 .eq. 1) then
         write(nout,'(a16, i2,(a))')  'qsm4 =', qsm4,
     &        ' : SM4 loops at NLO'
      elseif (model.eq.4 .and. qsm4 .eq. 2) then
         write(nout,'(a16, i2,(a))')  'qsm4 =', qsm4,
     &        ' : SM4 loops at NLO + leading 2-loop'
      endif
      write(nout,*)
      if(qsoftcoll.eq.1)then
         write(nout,'(a16,i2,(a))') 'qsoftcoll =',qsoftcoll
     &        ,' : subtraction method'
      elseif(qsoftcoll.eq.2)then
        write(nout,'(a16,i2,(a))') 'qsoftcoll =',qsoftcoll
     &        ,' : slicing method'
      endif
      if (qllfsr.eq.1.or.qllfsr.eq.3) then
         write(nout,'(a16,i2,2(a))') 'qllfsr =',qllfsr
     &        ,' : LL FSR up to O(alpha^3) included '
     &        ,'(with exponentiation)'
      elseif (qllfsr.eq.2.or.qllfsr.eq.4) then
         write(nout,'(a16,i2,2(a))') 'qllfsr =',qllfsr
     &        ,' : LL FSR up to O(alpha^3) included without '
     &        ,'exponentiation'
      elseif (qllfsr.eq.5) then
         write(nout,'(a16,i2,(a))') 'qllfsr =',qllfsr
     &        ,' : LL FSR up to O(alpha^2) included without '
     &        ,'exponentiation'
      elseif (qllfsr.eq.6) then
         write(nout,'(a16,i2,(a))') 'qllfsr =',qllfsr
     &        ,' : LL FSR up to O(alpha) included without '
     &        ,'exponentiation'
      endif
c photon recombination
      write(nout,*)
      if(qrecomb.eq.0)then
        write(nout,'(a16,i2,(a))')'qrecomb =',qrecomb,
     &        ' : no photon recombination applied'
        if(qrecombcolle.eq.1.and.qsoftcoll.eq.2) then
          write(nout,'(a16,i2,(a))')'qrecombcolle =',qrecombcolle,
     &        ' : only strictly collinear photons are recombined with electrons'
	endif
      elseif(qrecomb.eq.1)then
        write(nout,'(a16,i2,(a))')'qrecomb =',qrecomb,
     &        ' : photon recombination applied'
        write(nout,'(21x,"for invariant mass(f,ph) < ",f9.5," GeV")')
     &      precomb(1)
      endif
c separation cuts
      write(nout,*)
      if(qcuts.eq.0)then
CAM removed since cuts are not supported anyway
C        write(nout,'(a16,i2,(a))')'qcuts =',qcuts,
C     &        ' : no separation cuts'
      else
        write(nout,'(a16,i2,(a))')'qcuts =',qcuts,
     &        ': with separation cuts:'
        do i1=1,5
          if(ecut(i1).gt.0d0)then
            write(nout,'(20x,"energy(",i1,")  > ",f9.5," GeV")')
     &        i1,ecut(i1)
          endif
        enddo
        do i1=1,5
        do i2=i1+1,5
          if(ccut(i1,i2).lt.1d0)then
            write(nout,'(20x,"angle(",i1,",",i1,") > ",f9.5," deg")')
     &        i1,i2,dacos(ccut(i1,i2))/pi*180d0
          endif
        enddo
        enddo
        do i1=1,5
        do i2=i1+1,5
          if(scut(i1,i2).gt.0d0)then
            write(nout,'(20x,"mass(",i1,",",i1,")  > ",f9.5," GeV")')
     &        i1,i2,dsqrt(scut(i1,i2))
          endif
        enddo
        enddo
      endif
c other options/parameters
      if (smear.eq.1) then
         write(nout,'(a16,i2,(a))') 'smear =',smear
     &        ,' : smearing of histograms'
      endif
      if(contrib.eq.2)then
        write(nout,'(a16,f6.2,3x,(a))')'chww =',chww,
     &         ' : fitted constant for IBA'
        write(nout,'(a16,f6.2,3x,(a))')'chzz =',chzz,
     &         ' : fitted constant for IBA'
      endif
      write(nout,*)
c technical parameters
      if (qsoftcoll.eq.2) then
        write(nout,'(a16,g9.1,(a))')'DeltaE =',deltae,
     &       ' :  Slicing energy cut'
        write(nout,'(a16,g9.1,(a))')'DeltaTh =',deltaTh,
     &       ' :  Slicing angular cut'
      endif
c more detailed output
      if (moreout.eq.1) then
         if(qalp.eq.0)then
            write(nout,'(a16,i2,(a))') 'qalp =',qalp,
     &           ' : alpha(0)-scheme'
         elseif(qalp.eq.1)then
            write(nout,'(a16,i2,(a))') 'qalp =',qalp,
     &           ' : alpha_G_mu-scheme'
         elseif(qalp.eq.2)then
            write(nout,'(a16,i2,(a))') 'qalp =',qalp,
     &           ' : alpha(MZ)-scheme'
         endif
         if(qcomp.eq.0)then
            write(nout,'(a16,i2,(a))') 'qcomp =',qcomp
     &           ,' : NO complex mass scheme'
         elseif(qcomp.eq.1)then
            write(nout,'(a16,i2,(a))') 'qcomp =',qcomp
     &           ,' : complex mass scheme'
         endif
         write(nout,*)
         write(nout,*)'  calculate virtual correction every '
     &        ,nvirt(1),'(EW) ',nvirt(2),'(QCD) ','th time'
         write(nout,*)
         write(nout,'(a16,g9.1,(a))')'massmapping =',massmapping,
     &        ' :  parameter in h-function (mapping)'
         if (qsoftcoll.eq.1)
     &        write(nout,'(a16,g9.1,(a))')'mininvmass =',mininvmass,
     &        ' :  invariant-mass cut for real corrections'
         write(nout,'(a16,g9.1,(a))')'mffcutrel =',mffcutrel,
     &        ' :  ffbar invariant mass cut'
         if (qsoftcoll.eq.1)
     &        write(nout,'(a16,g9.1,(a))')'sub4cut =',sub4cut,
     &        ' :  same binning for 4-Phsp subtraction functions'
         if (qsoftcoll.eq.1)
     &        write(nout,'(a16,g9.1,(a))')'sub5cut =',sub5cut,
     &        ' :  same binning for 5-Phsp photon-subtr. functions'
         if (qsoftcoll.eq.1)
     &        write(nout,'(a16,g9.1,(a))')'sub5cutgl =',sub5cutgl,
     &        ' :  same binning for 5-Phsp gluon-subtr. functions'
         write(nout,*)
c         if (qfast.eq.0) then
c            write(nout,'(a16,i2,(a))')'qfast =',qfast,
c     &           ' : NO fast evaluation of tensor integrals'
c         elseif (qfast.eq.1) then
c            write(nout,'(a16,i2,(a))')'qfast =',qfast,
c     &           ' :  fast evaluation of tensor integrals'
c         endif
         if (rangen.eq.1) then
            write(nout,'(a16,i2,(a))')'rangen =',rangen,
     &           ' :  old Random number generator'
         elseif (rangen.eq.2) then
            write(nout,'(a16,i2,(a))')'rangen =',rangen,
     &           ' :  RCarry random number generator'
         elseif (rangen.eq.3) then
            write(nout,'(a16,i2,(a))')'rangen =',rangen,
     &           ' :  RANLUX random number generator'
         endif
         write(nout,'(a16,i2,(a))')'ranshift =',ranshift,
     &        ' :  different random variables'
         write(nout,*)
         write(nout,'(a16,g9.1,(a))')'lambda =',lambda,
     &        ' : Photon mass'
         write(nout,'(a16,g9.1,(a))')'mudim2 =',mudim2,
     &        ' : UV scale'
      endif
c narrow width approximation
c      write(nout,*)
c      write(nout,*)' Narrow width approximation (incl EW+QCD):'
c      write(nout,*)' -----------------------------------------'

      nwa = narrwidappr(1)
      nwaborn = narrwidappr(0)
      nwacorr = nwa - nwaborn

 2000 format((a),g13.6,(a))
c      write(nout,2000)'  Partial width: ',nwa,' MeV'
c      if (qwidth.eq.0) then
c         write(nout,2000)'           Born: ',nwaborn,
c     &        ' MeV (LO gauge-boson width)'
c      else
c         write(nout,2000)'           Born: ',nwaborn,
c     &        ' MeV (NLO gauge-boson width)'
c      endif
c      write(nout,2000)'     Correction: ',nwacorr,' MeV'
c      write(nout,*)

c Branching ratio includes QCD corrections
c Born: corrections included in denominator
c      if (nc.eq.1) then
c        br = width_Zff(q(1),q(2),gen(1),1,1)/gz
c        write(nout,*)' Branching ratio (Z->',fname(1)(1:length
c     &       (fname(1))),' ',fname(2)(1:length(fname(2))),') =',br
c        br = width_Zff(q(1),q(2),gen(1),0,1)/gz
c        write(nout,*)'   Born:',br
c        br = width_Zff(q(3),q(4),gen(3),1,1)/gz
c        write(nout,*)' Branching ratio (Z->',fname(3)(1:length(
c     &       fname(3))),' ',fname(4)(1:length(fname(4))),') =',br
c        br = width_Zff(q(3),q(4),gen(3),0,1)/gz
c        write(nout,*)'   Born:',br
c      elseif (cc.eq.1) then
c        br = width_Wff(q(1),q(2),1,1)/gw
c        write(nout,*)' Branching ratio (W->',fname(1)(1:length
c     &       (fname(1))),' ',fname(2)(1:length(fname(2))),') =',br
c        br = width_Wff(q(1),q(2),0,1)/gw
c        write(nout,*)'   Born:',br
c        br = width_Wff(q(3),q(4),1,1)/gw
c        write(nout,*)' Branching ratio (W->',fname(3)(1:length
c     &       (fname(3))),' ',fname(4)(1:length(fname(4))),') =',br
c        br = width_Wff(q(3),q(4),0,1)/gw
c        write(nout,*)'   Born:',br
c      endif

c date
cacb
c      call date_and_time(date,time,zone,values)
c      write(nout,*)
c      write(nout,44)date(1:4),date(5:6),date(7:8),time(1:2),time(3:4)
c     &              ,zone
c 44    FORMAT('started integration:',1x,(a),'-',(a),'-',(a),1x,(a),
c     &     ':',(a),1x,(a))
c      write(nout,*)

      if (maxout.gt.0) then
        write(nout,*)' Rejected events:'
        write(nout,*)' -----------------'
        write(nout,*)
      endif

      end

**************************************************************
*
*     write intermediate output
*
**************************************************************
      subroutine write_intermediate_output(av,si,n,ini)
      implicit none

      double precision average,sigma,av,si
      integer n,ini

      include 'COMMON/output.inc'

      If(ini.EQ.0) then
        write(nout,*) " Intermediate total results and errors in MeV:"
        write(nout2,*) " Intermediate total results and errors in MeV:"
	ini=1
      Endif

      average = av/dble(n)
      sigma   = si/dble(n)**2
      sigma   = dsqrt(abs(sigma-average**2/dble(n)))

      write(nout,*) n,average," +/-",sigma
      write(nout2,*) n,average," +/-",sigma

      end

**************************************************************
*
*     write final output
*
**************************************************************
      subroutine write_output(average,sigma,nout,ewcount,nancount,nminen,nmininv,
     &                    nsub4,nsub5,nsub5gl,qcdcount,moreout)
      implicit none
      include 'COMMON/max.inc'
      include 'COMMON/options.inc'
C      include 'COMMON/output.inc'
      integer nout,ewcount,nancount,nminen,nmininv
      integer nsub4,nsub5,nsub5gl,qcdcount,moreout
      integer i,ii,values(8),run_number
      character date*8,time*10,zone*5
      real*8 average,sigma
c sub contributions
      real*8 subav(maxsubc),subsi(maxsubc),submax(maxsubc)
      real*8 keep_av(maxfs),keep_si(maxfs)
      real*8 keep_subav(maxfs,maxsubc),keep_subsi(maxfs,maxsubc)
      integer subch(maxsubc),subn(maxsubc),subrej(maxw)
      common/subcontr/subav,subsi,submax,subch,subn,subrej
      common/summary_common/keep_av,keep_si,keep_subav,keep_subsi,
     &                      run_number


      if (moreout.eq.1) then
         if (qsoftcoll.eq.1) then
            write(nout,'(a27,i8,a17)')' inv.mass < mininvmass: ',nmininv
     &           ,' events discarded'
            write(nout,'(a27,i8,a17)')' 1-z < sub4cut: ',nsub4
     &           ,' events discarded'
            write(nout,'(a27,i8,a17)')' E_gamma,y < sub5cut: ',nsub5
     &           ,' events discarded (multiple counting)'
            write(nout,'(a27,i8,a17)')' E_gluon,y < sub5cutgl: ',nsub5gl
     &           ,' events discarded (multiple counting)'
         endif
         write(nout,'(a27,i8,a17)')' energy < min. energy: ',nminen
     &        ,' events discarded'
         write(nout,*)'   (double counting possible)'
         do i=1,maxw
            if (subrej(i).ne.0) then
               write(nout,'((a),i2,(a),f6.2,(a))')'    #',i,': '
     &              ,dfloat(subrej(i))/nevents*100d0,'%'
            endif
         enddo
      endif
 1000 format((a),f18.10," MeV")
 1005 format("    event number",i10,", channel",i3)
      write(nout,*)
      write(nout,*)' Results:'
      write(nout,*)' -------'
      write(nout,*)
      write(nout,1000)'  Average             = ',average
      write(nout,1000)'  Standard deviation  = ',sigma
      write(nout,*)
      write(nout,*)' Subcontributions:'
      write(nout,*)' -----------------'
      if (qwidth.eq.0) then
         write(nout,*)' Born cross section (LO gauge-',
     &        'boson width):'
      else
         write(nout,*)' Born cross section (NLO gauge-',
     &        'boson width):'
      endif
      write(nout,1000)'   Average            = ',subav(1)
      write(nout,1000)'   Standard deviation = ',subsi(1)
      if (debug.ne.0) then
        write(nout,1000)'   largest event      = ',submax(1)
        write(nout,1005)subn(1),subch(1)
      endif
      if (subav(2).ne.0d0 .or. subav(4).ne.0d0) then
        write(nout,*)
        write(nout,*)' EW corrections:'
        write(nout,1000)'   Average            = ',subav(2) + subav(4)
        write(nout,1000)'   Standard deviation = ',
     &       dsqrt(subsi(2)**2 + subsi(4)**2)
      endif
      if (subav(3).ne.0d0 .and. subav(5).ne.0d0) then
        write(nout,*)
        write(nout,*)' QCD corrections:'
        write(nout,1000)'   Average            = ',subav(3) + subav(5)
        write(nout,1000)'   Standard deviation = ',
     &       dsqrt(subsi(3)**2 + subsi(5)**2)
      endif
      if (subav(2).ne.0d0 .and. debug.ne.0) then
        write(nout,*)
        write(nout,*)' EW Virtual corrections:'
        write(nout,1000)'   Average            = ',subav(2)
        write(nout,1000)'   Standard deviation = ',subsi(2)
        write(nout,1000)'   largest event      = ',submax(2)
        write(nout,1005)subn(2),subch(2)
      endif
      if (subav(3).ne.0d0 .and. debug.ne.0) then
        write(nout,*)
        write(nout,*)' QCD Virtual corrections:'
        write(nout,1000)'   Average            = ',subav(3)
        write(nout,1000)'   Standard deviation = ',subsi(3)
        write(nout,1000)'   largest event      = ',submax(3)
        write(nout,1005)subn(3),subch(3)
      endif
      if (subav(4).ne.0d0 .and. debug.ne.0) then
        write(nout,*)
        write(nout,*)' Photon Bremsstrahlung:'
        write(nout,1000)'   Average            = ',subav(4)
        write(nout,1000)'   Standard deviation = ',subsi(4)
        write(nout,1000)'   largest event      = ',submax(4)
        write(nout,1005)subn(4),subch(4)
      endif
      if (subav(5).ne.0d0 .and. debug.ne.0) then
        write(nout,*)
        write(nout,*)' Gluon Bremsstrahlung:'
        write(nout,1000)'   Average            = ',subav(5)
        write(nout,1000)'   Standard deviation = ',subsi(5)
        write(nout,1000)'   largest event      = ',submax(5)
        write(nout,1005)subn(5),subch(5)
      endif
      if (debug .ne. 0) then
         write(nout,'(a,i10)') 'nancount = ', nancount
         write(nout,'(a,i10)') 'ewcount  = ', ewcount
         write(nout,'(a,i10)') 'qcdcount = ', qcdcount
      endif
      
      write(nout,*)
	
      keep_av(run_number)=average
      keep_si(run_number)=sigma
      do ii=1,maxsubc
         keep_subav(run_number,ii)=subav(ii)
	 keep_subsi(run_number,ii)=subsi(ii)
      enddo

c date
cacb
c      call date_and_time(date,time,zone,values)
c      write(nout,*)
c      write(nout,44)date(1:4),date(5:6),date(7:8),time(1:2),time(3:4)
c     &              ,zone
c 44    FORMAT('finished weighted events:',1x,(a),'-',(a),'-',(a),1x,(a),
c     &     ':',(a),1x,(a))

      end

**************************************************************
*
*     write summary output
*
**************************************************************
      subroutine summary_output(nout)
      implicit none
      include 'COMMON/max.inc'
      include 'COMMON/options.inc'
C      include 'COMMON/output.inc'
      include 'COMMON/processpar.inc'
      integer nout
      integer ii,kk,run_number,length
      real*8 av,si,tot_subav(maxsubc),tot_subsi(maxsubc)
      real*8 keep_av(maxfs),keep_si(maxfs)
      real*8 keep_subav(maxfs,maxsubc),keep_subsi(maxfs,maxsubc)
      common/summary_common/keep_av,keep_si,keep_subav,keep_subsi,
     &                      run_number

      av=0D0
      si=0D0
      do kk=1,maxsubc
	 tot_subav(kk)=0D0
	 tot_subsi(kk)=0D0
      enddo
      Write(nout,*) ""
      Write(nout,*) "Summary for the different channels:"
      Write(nout,*) "-----------------------------------"
      Write(nout,*) ""
      do ii=1,count_final(1)
         write(nout,*)'Process:  H ->   '
     &    ,fnameinput(1,ii)(1:length(fnameinput(1,ii))),' '
     &    ,fnameinput(2,ii)(1:length(fnameinput(2,ii))),' '
     &    ,fnameinput(3,ii)(1:length(fnameinput(3,ii))),' '
     &    ,fnameinput(4,ii)(1:length(fnameinput(4,ii))),
     &     ' (+Photon/Gluon)'
         write(nout,1000)'  Average             = ',keep_av(ii)
         write(nout,1000)'  Standard deviation  = ',keep_si(ii)
         av=av+keep_av(ii)
	 si=si+keep_si(ii)**2
	 do kk=1,maxsubc
	    tot_subav(kk)=tot_subav(kk)+keep_subav(ii,kk)
	    tot_subsi(kk)=tot_subsi(kk)+keep_subsi(ii,kk)**2
	 enddo
      enddo
      If(special.EQ.0) Then
         Write(nout,*) ""
         Write(nout,*) "Combined width for all calculated channels:"
         Write(nout,*) "-------------------------------------------"
         Write(nout,*) ""
         call write_summary(av,si,tot_subav,tot_subsi,nout)
      Else If(special.EQ.1) then
	 Write(nout,*) ""
         Write(nout,*) "Total width of the Higgs boson:"
         Write(nout,*) "-------------------------------"
         Write(nout,*) ""
         call combine_for_total_width(av,si,tot_subav,tot_subsi)
         call write_summary(av,si,tot_subav,tot_subsi,nout)
	 Write(nout,*) ""
         Write(nout,*) "Gamma_WW:"
         Write(nout,*) "---------"
         Write(nout,*) ""
         call combine_for_WW_width(av,si,tot_subav,tot_subsi)
         call write_summary(av,si,tot_subav,tot_subsi,nout)
	 Write(nout,*) ""
         Write(nout,*) "Gamma_ZZ:"
         Write(nout,*) "---------"
         Write(nout,*) ""
         call combine_for_ZZ_width(av,si,tot_subav,tot_subsi)
         call write_summary(av,si,tot_subav,tot_subsi,nout)
	 Write(nout,*) ""
         Write(nout,*) "Gamma_Int:"
         Write(nout,*) "----------"
         Write(nout,*) ""
         call combine_for_Int_width(av,si,tot_subav,tot_subsi)
         call write_summary(av,si,tot_subav,tot_subsi,nout)
      Else If(special.EQ.2) then
	 Write(nout,*) ""
         Write(nout,*) "Leptonic width of the Higgs boson:"
         Write(nout,*) "-------------------------------"
         Write(nout,*) ""
         call combine_for_leptonic_width(av,si,tot_subav,tot_subsi)
         call write_summary(av,si,tot_subav,tot_subsi,nout)
      Else If(special.EQ.3) then
	 Write(nout,*) ""
         Write(nout,*) "Semi-leptonic width of the Higgs boson:"
         Write(nout,*) "-------------------------------"
         Write(nout,*) ""
         call combine_for_semi_leptonic_width(av,si,tot_subav,tot_subsi)
         call write_summary(av,si,tot_subav,tot_subsi,nout)
      Else If(special.EQ.4) then
	 Write(nout,*) ""
         Write(nout,*) "Hadronic width of the Higgs boson:"
         Write(nout,*) "-------------------------------"
         Write(nout,*) ""
         call combine_for_hadronic_width(av,si,tot_subav,tot_subsi)
         call write_summary(av,si,tot_subav,tot_subsi,nout)
      Else If(special.EQ.5) then
	 Write(nout,*) ""
         Write(nout,*) "Gamma_WW:"
         Write(nout,*) "-------------------------------"
         Write(nout,*) ""
         call combine_for_WW_width_only(av,si,tot_subav,tot_subsi)
         call write_summary(av,si,tot_subav,tot_subsi,nout)
      Else If(special.EQ.6) then
	 Write(nout,*) ""
         Write(nout,*) "Gamma_ZZ:"
         Write(nout,*) "-------------------------------"
         Write(nout,*) ""
         call combine_for_ZZ_width_only(av,si,tot_subav,tot_subsi)
         call write_summary(av,si,tot_subav,tot_subsi,nout)
      Else If(special.EQ.7) then
	 Write(nout,*) ""
         Write(nout,*) "Gamma_Int:"
         Write(nout,*) "-------------------------------"
         Write(nout,*) ""
         call combine_for_Int_width_only(av,si,tot_subav,tot_subsi)
         call write_summary(av,si,tot_subav,tot_subsi,nout)
      Endif
 1000 format((a),f18.10," MeV")

      end

************************************************************************
*
*     write output of unweighting procedure
*
************************************************************************
      subroutine write_unweighting(genwev,novershoot,glomaxweight,
     &                             nout,genunwbornev,genunwev,negev)
      implicit none
      include 'COMMON/options.inc'
C      include 'COMMON/output.inc'
      include 'COMMON/max.inc'
      integer nout,genunwbornev,genunwev,negev
      integer values(8)
      character date*8,time*10,zone*5
      integer genwev,novershoot
      real*8 glomaxweight

      if (nunwevents.eq.0) return
      write(nout,*)
      write(nout,*)' Unweighted events:'
      write(nout,*)' ------------------'
      if (genunwev.lt.nunwevents) write(nout,*) ' only ', genunwev,
     &     ' unweighted events, unweighting procedure inefficient'

      write(nout,'((a),i8,(a))') ' generated ',genunwev,
     &     ' unweighted events (including negative events)'
C      If(genunwbornev.GT.nunwevents) genunwbornev=nunwevents
      write(nout,'((a),i8,(a))') ' generated ',genunwbornev,
     &     ' unweighted born events'
      write(nout,'((a),i11,(a))')'   by generating weighted events ',
     &     genwev,' times'
      write(nout,'((a),(a),i4)') ' number of weights larger than ',
     &     'largest weight from 1st cycle: ',novershoot
      write(nout,'((a),i8)') ' number of negative unweighted events: ',
     &     negev
      write(nout,'((a),(a),g12.5)') ' maximal weight used for',
     &     ' generating unweighted events: ',glomaxweight

      write(nout,*)

c date
cacb
c      call date_and_time(date,time,zone,values)
c      write(nout,*)
c      write(nout,44)date(1:4),date(5:6),date(7:8),time(1:2),time(3:4)
c     &              ,zone
c 44    FORMAT('finished unweighted events:',1x,(a),'-',(a),'-',(a),1x,
c     &     (a),':',(a),1x,(a))

C      close(nout)

      end

******************************************************************
      INTEGER FUNCTION LENGTH(STRING)
*Returns length of string ignoring trailing blanks
******************************************************************
      CHARACTER*(*) STRING
      DO 35, I = LEN(STRING), 1, -1
         IF(STRING(I:I) .NE. ' ') GO TO 40
35    CONTINUE
40    LENGTH = I
      END

******************************************************************
      INTEGER FUNCTION STARTPOS(STRING)
*Returns first non-blank character in a string
******************************************************************
      CHARACTER*(*) STRING
      DO 35, I = 1, LEN(STRING)
         IF(STRING(I:I) .NE. ' ') GO TO 40
35    CONTINUE
40    STARTPOS = I
      END

**************************************************************
*
*     organize_process
*
*  translate input into well defined processes enforcing the
*  conventions in the rest of the program
*
**************************************************************
      subroutine organize_process
      implicit none

      integer fs,ii,jj,kk,proper,Wch(2),Zch(2),doublet(2)
      integer whichin,whichout
      character*8 part(2),anti(2)
      include 'COMMON/processpar.inc'
      include 'COMMON/output.inc'
      include 'COMMON/max.inc'
      integer iNumProcs, iErr, iMyRank
      common/mpistuff/iNumProcs,iMyRank

      special=0

      do whichin=1,count_final(0)
        If(count_final(1).LE.maxfs) Then
	   count_final(1)=count_final(1)+1
	else
	   write(nout,*)
     &           'too many channels requested: 100 Max  '
           write(nout,*) 'The program will be stopped here.'
	   close(nout)
	   Stop
	Endif
        call read_channel(whichin, count_final(1))
      enddo

      If(special.EQ.1) then
         call prepare_for_total_width()
	 count_final(1)=19
	 If(iMyRank.EQ.0) write(nout,*)
     &      'Prophecy will calculate the total Higgs width.'
	 If(iMyRank.EQ.0) write(nout2,*)
     &      'Prophecy will calculate the total Higgs width.'
      Else If(special.EQ.2) then
         call prepare_for_leptonic_width()
	 count_final(1)=7
	 If(iMyRank.EQ.0) write(nout,*)
     &      'Prophecy will calculate the leptonic Higgs width.'
	 If(iMyRank.EQ.0) write(nout2,*)
     &      'Prophecy will calculate the leptonic Higgs width.'
      Else If(special.EQ.3) then
         call prepare_for_semi_leptonic_width()
	 count_final(1)=5
	 If(iMyRank.EQ.0) write(nout,*)
     &      'Prophecy will calculate the semi-leptonic Higgs width.'
	 If(iMyRank.EQ.0) write(nout2,*)
     &      'Prophecy will calculate the semi-leptonic Higgs width.'
      Else If(special.EQ.4) then
         call prepare_for_hadronic_width()
	 count_final(1)=7
	 If(iMyRank.EQ.0) write(nout,*)
     &      'Prophecy will calculate the hadronic Higgs width.'
	 If(iMyRank.EQ.0) write(nout2,*)
     &      'Prophecy will calculate the hadronic Higgs width.'
      Else If(special.EQ.5) then
         call prepare_for_WW_width()
	 count_final(1)=3
	 If(iMyRank.EQ.0) write(nout,*)
     &      'Prophecy will calculate Gamma_WW.'
	 If(iMyRank.EQ.0) write(nout2,*)
     &      'Prophecy will calculate Gamma_WW.'
      Else If(special.EQ.6) then
         call prepare_for_ZZ_width()
	 count_final(1)=14
	 If(iMyRank.EQ.0) write(nout,*)
     &      'Prophecy will calculate Gamma_ZZ.'
	 If(iMyRank.EQ.0) write(nout2,*)
     &      'Prophecy will calculate Gamma_ZZ.'
      Else If(special.EQ.7) then
         call prepare_for_Int_width()
	 count_final(1)=6
	 If(iMyRank.EQ.0) write(nout,*)
     &      'Prophecy will calculate Gamma_Int.'
	 If(iMyRank.EQ.0) write(nout2,*)
     &      'Prophecy will calculate Gamma_Int.'
      Endif

      do fs=1,count_final(1)

	 jj=1
	 kk=1
	 do ii=1,4
           If(fnameinput(ii,fs)(1:4).EQ."anti") then
	     If(fnameinput(ii,fs)(1:8).NE.'anti-uq ' .AND.
     &          fnameinput(ii,fs)(1:8).NE.'anti-dq ' .AND.
     &          fnameinput(ii,fs)(1:8).NE.'anti-cq ' .AND.
     &          fnameinput(ii,fs)(1:8).NE.'anti-sq ' .AND.
     &          fnameinput(ii,fs)(1:8).NE.'anti-e  ' .AND.
     &          fnameinput(ii,fs)(1:8).NE.'anti-mu ' .AND.
     &          fnameinput(ii,fs)(1:8).NE.'anti-nue' .AND.
     &          fnameinput(ii,fs)(1:8).NE.'anti-num' ) Then
                write(nout,*) 'final state particle ',
     &                         fnameinput(ii,fs)(1:8),'not known'
        	write(nout,*) 'The program will be stopped here.'
		close(nout)
		Stop
	     Endif
	     anti(jj)=fnameinput(ii,fs)
	     jj=jj+1
	     If(jj.EQ.4) Then
               write(nout,*) 'More than 2 anti-fermions in final state.'
               write(nout,*) 'The program will be stopped here.'
               If(nout.ne.6) Then
	         write(6,*) 'More than 2 anti-fermions in final state.'
                 write(6,*) 'The program will be stopped here.'
	       Endif
	       close(nout)
	       Stop
	     Endif
	   Else
	     If(fnameinput(ii,fs)(1:8).NE.'uq      ' .AND.
     &          fnameinput(ii,fs)(1:8).NE.'dq      ' .AND.
     &          fnameinput(ii,fs)(1:8).NE.'cq      ' .AND.
     &          fnameinput(ii,fs)(1:8).NE.'sq      ' .AND.
     &          fnameinput(ii,fs)(1:8).NE.'e       ' .AND.
     &          fnameinput(ii,fs)(1:8).NE.'mu      ' .AND.
     &          fnameinput(ii,fs)(1:8).NE.'nue     ' .AND.
     &          fnameinput(ii,fs)(1:8).NE.'num     ' ) Then
                write(nout,*) 'final state particle ',
     &                         fnameinput(ii,fs)(1:8),' not known'
        	write(nout,*) 'The program will be stopped here.'
                If(nout.ne.6) Then
                  write(6,*) 'final state particle ',
     &                         fnameinput(ii,fs)(1:8),' not known'
        	  write(6,*) 'The program will be stopped here.'
		Endif
		close(nout)
		Stop
	     Endif
	     part(kk)=fnameinput(ii,fs)
	     kk=kk+1
	     If(kk.EQ.4) Then
               write(nout,*) 'Less than 2 anti-fermions in final state.'
               write(nout,*) 'The program will be stopped here.'
               If(nout.ne.6) Then
                 write(6,*) 'Less than 2 anti-fermions in final state.'
                 write(6,*) 'The program will be stopped here.'
               Endif
	       close(nout)
	       Stop
	     Endif
	   Endif
	 enddo

	 do jj=1,2
	 Wch(jj)=0
	 Zch(jj)=0
	 do ii=1,2
	   If(part(jj).EQ.'e') Then
              doublet(jj)=-1
	      If(anti(ii).EQ.'anti-nue' .AND. Wch(1).NE.ii) then
		 Wch(jj)=ii
	      Endif
	      If(anti(ii).EQ.'anti-e' .AND. Zch(1).NE.ii) then
		 Zch(jj)=ii
	      Endif
	   Else If(part(jj).EQ.'mu') Then
              doublet(jj)=-1
              If(anti(ii).EQ.'anti-num' .AND. Wch(1).NE.ii) then
		 Wch(jj)=ii
	      Endif
	      If(anti(ii).EQ.'anti-mu' .AND. Zch(1).NE.ii) then
		 Zch(jj)=ii
              Endif
	   Else If(part(jj).EQ.'nue') Then
              doublet(jj)=1
              If(anti(ii).EQ.'anti-e' .AND. Wch(1).NE.ii) then
		 Wch(jj)=ii
	      Endif
	      If(anti(ii).EQ.'anti-nue' .AND. Zch(1).NE.ii) then
        	 Zch(jj)=ii
	      Endif
	   Else If(part(jj).EQ.'num') Then
              doublet(jj)=1
              If(anti(ii).EQ.'anti-mu' .AND. Wch(1).NE.ii) then
		 Wch(jj)=ii
	      Endif
	      If(anti(ii).EQ.'anti-num' .AND. Zch(1).NE.ii) then
        	 Zch(jj)=ii
	      Endif
	   Else If(part(jj).EQ.'dq') Then
              doublet(jj)=-1
              If(anti(ii).EQ.'anti-uq' .AND. Wch(1).NE.ii) then
		 Wch(jj)=ii
	      Endif
	      If(anti(ii).EQ.'anti-dq' .AND. Zch(1).NE.ii) then
        	 Zch(jj)=ii
	      Endif
	   Else If(part(jj).EQ.'uq') Then
              doublet(jj)=1
              If(anti(ii).EQ.'anti-dq' .AND. Wch(1).NE.ii) then
		 Wch(jj)=ii
	      Endif
	      If(anti(ii).EQ.'anti-uq' .AND. Zch(1).NE.ii) then
        	 Zch(jj)=ii
	      Endif
	   Else If(part(jj).EQ.'sq') Then
              doublet(jj)=-1
              If(anti(ii).EQ.'anti-cq' .AND. Wch(1).NE.ii) then
		 Wch(jj)=ii
	      Endif
	      If(anti(ii).EQ.'anti-sq' .AND. Zch(1).NE.ii) then
        	 Zch(jj)=ii
	      Endif
	   Else If(part(jj).EQ.'cq') Then
              doublet(jj)=1
              If(anti(ii).EQ.'anti-sq' .AND. Wch(1).NE.ii) then
		 Wch(jj)=ii
	      Endif
	      If(anti(ii).EQ.'anti-cq' .AND. Zch(1).NE.ii) then
		 Zch(jj)=ii
	      Endif
	   Else
              write(nout,*) 'Unknown fermion in input'
              write(nout,*) 'The program will be stopped here.'
              If(nout.ne.6) Then
                write(6,*) 'Unknown fermion in input'
                write(6,*) 'The program will be stopped here.'
	      Endif
	      close(nout)
	      Stop
	   Endif
	 Enddo
	 Enddo

	 proper=1
	 If(Wch(1).EQ.1) Then
            If(Wch(2).EQ.2) Then
	       If(doublet(1).EQ.1 .AND. doublet(2).EQ.-1) Then
		  fnameinput(1,fs)=part(1)
		  fnameinput(2,fs)=anti(1)
		  fnameinput(3,fs)=part(2)
		  fnameinput(4,fs)=anti(2)
	       Else If(doublet(1).EQ.-1 .AND. doublet(2).EQ.1) Then
		  fnameinput(1,fs)=part(2)
		  fnameinput(2,fs)=anti(2)
		  fnameinput(3,fs)=part(1)
		  fnameinput(4,fs)=anti(1)
	       Else
		  proper=0
		  Write(nout,*) 'Two same-charge Ws in final state.'
	       Endif
	    Else
	       proper=0
	       Write(nout,*) 'Final state fermions cannot form 2 '//
     &                    'W/Z Bosons (unit CKM matrix)'
	    Endif
	 Else If(Wch(1).EQ.2) Then
            If(Wch(2).EQ.1) Then
	       If(doublet(1).EQ.1 .AND. doublet(2).EQ.-1) Then
		  fnameinput(1,fs)=part(1)
		  fnameinput(2,fs)=anti(2)
		  fnameinput(3,fs)=part(2)
		  fnameinput(4,fs)=anti(1)
	       Else If(doublet(1).EQ.-1 .AND. doublet(2).EQ.1) Then
		  fnameinput(1,fs)=part(2)
		  fnameinput(2,fs)=anti(1)
		  fnameinput(3,fs)=part(1)
		  fnameinput(4,fs)=anti(2)
	       Else
		  proper=0
		  Write(nout,*) 'Two same-charge Ws in final state.'
	       Endif
	    Else
	       proper=0
	       Write(nout,*) 'Final state fermions cannot form 2 '//
     &                    'W/Z Bosons (unit CKM matrix)'
	    Endif
	 Else If(Zch(1).EQ.1) Then
            If(Zch(2).EQ.2) Then
	       If(doublet(1).EQ.1 .or. doublet(2).eq.-1) Then
		  fnameinput(1,fs)=part(1)
		  fnameinput(2,fs)=anti(1)
		  fnameinput(3,fs)=part(2)
		  fnameinput(4,fs)=anti(2)
	       Else
		  fnameinput(1,fs)=part(2)
		  fnameinput(2,fs)=anti(2)
		  fnameinput(3,fs)=part(1)
		  fnameinput(4,fs)=anti(1)
	       Endif
	    Else
	       proper=0
	       Write(nout,*) 'Final state fermions cannot form 2 '//
     &                    'W/Z Bosons (unit CKM matrix)'
	    Endif
	 Else If(Zch(1).EQ.2) Then
            If(Zch(2).EQ.1) Then
	       If(doublet(1).EQ.1 .or. doublet(2).eq.-1) Then
		  fnameinput(1,fs)=part(1)
		  fnameinput(2,fs)=anti(2)
		  fnameinput(3,fs)=part(2)
		  fnameinput(4,fs)=anti(1)
	       Else
		  fnameinput(1,fs)=part(2)
		  fnameinput(2,fs)=anti(1)
		  fnameinput(3,fs)=part(1)
		  fnameinput(4,fs)=anti(2)
	       Endif
	    Else
	       proper=0
	       Write(nout,*) 'Final state fermions cannot form 2 '//
     &                    'W/Z Bosons (unit CKM matrix)'
	    Endif
	 Else
            write(nout,*) 'Final state fermions cannot form W/Z '//
     &                 'Bosons (unit CKM matrix)'
            proper=0
	 Endif

	 If(proper.EQ.0) Then
            write(nout,*) 'Problems with the chosen final state.'
            write(nout,*) 'The program will be stopped here.'
            If(nout.ne.6) Then
              write(6,*) 'Problems with the chosen final state.'
              write(6,*) 'The program will be stopped here.'
	    Endif
	    close(nout)
	    Stop
	 Endif

      enddo

      If(count_final(1).EQ.0) Then
C  Fall back on default if no input in inputfile
         count_final(1)=1
	 fnameinput(1,1)=fname(1)
	 fnameinput(2,1)=fname(2)
	 fnameinput(3,1)=fname(3)
	 fnameinput(4,1)=fname(4)
      Endif

      If(iMyRank.EQ.0) Then
        If(count_final(1).GT.1) Then
          Write(nout,*)
     &          'Prophecy will run for the following final states:'
          Write(nout2,*)
     &          'Prophecy will run for the following final states:'
          do fs=1,count_final(1)
	     Write(nout,*) 'H -> ',fnameinput(1,fs)(1:3),' '
     &                            ,fnameinput(2,fs)(1:8),' '
     &                            ,fnameinput(3,fs)(1:3),' '
     &                            ,fnameinput(4,fs)(1:8)
	     Write(nout2,*) 'H -> ',fnameinput(1,fs)(1:3),' '
     &                            ,fnameinput(2,fs)(1:8),' '
     &                            ,fnameinput(3,fs)(1:3),' '
     &                            ,fnameinput(4,fs)(1:8)
	  enddo
        Endif
      Endif

      end

**************************************************************
*
*     read_channel
*
*  translates input line into fname variable
*
**************************************************************
      subroutine read_channel(whichin,whichout)
      implicit none
      include 'COMMON/processpar.inc'
      include 'COMMON/output.inc'
      character*100 input
      integer count,start,ii,whichin,whichout

      count=0
      start=0

      input=finalstateinput(whichin)

      do ii=1,100
         If(input(ii:ii).NE.' '.AND. start.EQ.0) Then
	    start=ii
	 Else If(input(ii:ii).EQ.' '.AND. start.NE.0) Then
	    count=count+1
	    If(count.LE.4) Then
	      If(ii-start.LE.8) Then
	        fnameinput(count,whichout)=input(start:ii-1)
	      Else
		fnameinput(count,whichout)=input(start:start+7)
	      Endif
	    Else
	      Write(nout,*) 'Too many final state particles in'
	      Write(nout,*) 'channel= ',input
              Write(nout,*) 'The program will be stopped here.'
	      If(nout.ne.6) Then
	        Write(6,*) 'Too many final state particles in'
	        Write(6,*) 'channel= ',input
                Write(6,*) 'The program will be stopped here.'
	      Endif
	      close(nout)
	      Stop
	    Endif
	    start=0
	 Else If(input(ii:ii).EQ.'!') Then
	    goto 99
	 Endif
      enddo
 99   continue

      If(fnameinput(1,whichout).EQ.'total') then
         special=1
	 Return
      Else If(fnameinput(1,whichout).EQ.'leptonic') then
         special=2
	 Return
      Else If(fnameinput(1,whichout).EQ.'semi-lep') then
         special=3
	 Return
      Else If(fnameinput(1,whichout).EQ.'hadronic') then
         special=4
	 Return
      Else If(fnameinput(1,whichout).EQ.'WW') then
         special=5
	 Return
      Else If(fnameinput(1,whichout).EQ.'ZZ') then
         special=6
	 Return
      Else If(fnameinput(1,whichout).EQ.'interfer') then
         special=7
	 Return
      Endif

      If(count.LT.4) Then
	 Write(nout,*) 'Too few final state particles in'
	 Write(nout,*) 'channel= ',input
         Write(nout,*) 'The program will be stopped here.'
	 close(nout)
	 Stop
      Endif

      end

**************************************************************
*
*     prepare_for_total_width
*
*  setup all channels for calculation of the total width
*
**************************************************************
      subroutine prepare_for_total_width()
      implicit none

      integer whichin
      include 'COMMON/processpar.inc'

      finalstateinput(1)='nue anti-nue num anti-num'
      finalstateinput(2)='e   anti-e   mu  anti-mu'
      finalstateinput(3)='nue anti-nue mu  anti-mu'
      finalstateinput(4)='nue anti-e   mu  anti-num'
      finalstateinput(5)='nue anti-nue nue anti-nue'
      finalstateinput(6)='e   anti-e   e   anti-e'
      finalstateinput(7)='nue anti-e   e   anti-nue'
      finalstateinput(8)='uq anti-uq cq anti-cq'
      finalstateinput(9)='dq anti-dq sq anti-sq'
      finalstateinput(10)='uq anti-uq sq anti-sq'
      finalstateinput(11)='uq anti-dq sq anti-cq'
      finalstateinput(12)='uq anti-uq uq anti-uq'
      finalstateinput(13)='dq anti-dq dq anti-dq'
      finalstateinput(14)='uq anti-dq dq anti-uq'
      finalstateinput(15)='nue anti-nue uq anti-uq'
      finalstateinput(16)='nue anti-nue dq anti-dq'
      finalstateinput(17)='uq anti-uq e anti-e'
      finalstateinput(18)='dq anti-dq e anti-e'
      finalstateinput(19)='nue anti-e dq anti-uq'

      do whichin=1,19
        call read_channel(whichin, whichin)
      enddo

      end

**************************************************************
*
*     combine_for_total_width
*
*  add up results for different channels to calculate total width
*
**************************************************************
      Subroutine combine_for_total_width(av,si,tot_subav,tot_subsi)
      implicit none

      include 'COMMON/max.inc'

      integer run_number,ii
      real*8 av,si,tot_subav(maxsubc),tot_subsi(maxsubc)
      real*8 keep_av(maxfs),keep_si(maxfs)
      real*8 keep_subav(maxfs,maxsubc),keep_subsi(maxfs,maxsubc)
      common/summary_common/keep_av,keep_si,keep_subav,keep_subsi,
     &                      run_number

      av=  3*keep_av(1)
     &   + 3*keep_av(2)
     &   + 6*keep_av(3)
     &   + 6*keep_av(4)
     &   + 3*keep_av(5)
     &   + 3*keep_av(6)
     &   + 3*keep_av(7)
     &   +   keep_av(8)
     &   + 3*keep_av(9)
     &   + 4*keep_av(10)
     &   + 2*keep_av(11)
     &   + 2*keep_av(12)
     &   + 3*keep_av(13)
     &   + 2*keep_av(14)
     &   + 6*keep_av(15)
     &   + 9*keep_av(16)
     &   + 6*keep_av(17)
     &   + 9*keep_av(18)
     &   + 12*keep_av(19)
      si=  (3*keep_si(1))**2
     &   + (3*keep_si(2))**2
     &   + (6*keep_si(3))**2
     &   + (6*keep_si(4))**2
     &   + (3*keep_si(5))**2
     &   + (3*keep_si(6))**2
     &   + (3*keep_si(7))**2
     &   + (  keep_si(8))**2
     &   + (3*keep_si(9))**2
     &   + (4*keep_si(10))**2
     &   + (2*keep_si(11))**2
     &   + (2*keep_si(12))**2
     &   + (3*keep_si(13))**2
     &   + (2*keep_si(14))**2
     &   + (6*keep_si(15))**2
     &   + (9*keep_si(16))**2
     &   + (6*keep_si(17))**2
     &   + (9*keep_si(18))**2
     &   + (12*keep_si(19))**2

      do ii=1,maxsubc
      tot_subav(ii)=  3*keep_subav(1,ii)
     &              + 3*keep_subav(2,ii)
     &              + 6*keep_subav(3,ii)
     &              + 6*keep_subav(4,ii)
     &              + 3*keep_subav(5,ii)
     &              + 3*keep_subav(6,ii)
     &              + 3*keep_subav(7,ii)
     &              +	keep_subav(8,ii)
     &              + 3*keep_subav(9,ii)
     &              + 4*keep_subav(10,ii)
     &              + 2*keep_subav(11,ii)
     &              + 2*keep_subav(12,ii)
     &              + 3*keep_subav(13,ii)
     &              + 2*keep_subav(14,ii)
     &              + 6*keep_subav(15,ii)
     &              + 9*keep_subav(16,ii)
     &              + 6*keep_subav(17,ii)
     &              + 9*keep_subav(18,ii)
     &              + 12*keep_subav(19,ii)
      tot_subsi(ii)=  (3*keep_subsi(1,ii))**2
     &              + (3*keep_subsi(2,ii))**2
     &              + (6*keep_subsi(3,ii))**2
     &              + (6*keep_subsi(4,ii))**2
     &              + (3*keep_subsi(5,ii))**2
     &              + (3*keep_subsi(6,ii))**2
     &              + (3*keep_subsi(7,ii))**2
     &              + (  keep_subsi(8,ii))**2
     &              + (3*keep_subsi(9,ii))**2
     &              + (4*keep_subsi(10,ii))**2
     &              + (2*keep_subsi(11,ii))**2
     &              + (2*keep_subsi(12,ii))**2
     &              + (3*keep_subsi(13,ii))**2
     &              + (2*keep_subsi(14,ii))**2
     &              + (6*keep_subsi(15,ii))**2
     &              + (9*keep_subsi(16,ii))**2
     &              + (6*keep_subsi(17,ii))**2
     &              + (9*keep_subsi(18,ii))**2
     &              + (12*keep_subsi(19,ii))**2
      enddo

      end

**************************************************************
*
*     combine_for_WW_width
*
*  add up results for different channels to calculate WW width
*
**************************************************************
      Subroutine combine_for_WW_width(av,si,tot_subav,tot_subsi)
      implicit none

      include 'COMMON/max.inc'

      integer run_number,ii
      real*8 av,si,tot_subav(maxsubc),tot_subsi(maxsubc)
      real*8 keep_av(maxfs),keep_si(maxfs)
      real*8 keep_subav(maxfs,maxsubc),keep_subsi(maxfs,maxsubc)
      common/summary_common/keep_av,keep_si,keep_subav,keep_subsi,
     &                      run_number

      av=  9*keep_av(4)
     &   + 4*keep_av(11)
     &   + 12*keep_av(19)
      si=  (9*keep_si(4))**2
     &   + (4*keep_si(11))**2
     &   + (12*keep_si(19))**2

      do ii=1,maxsubc
      tot_subav(ii)=  9*keep_subav(4,ii)
     &              + 4*keep_subav(11,ii)
     &              + 12*keep_subav(19,ii)
      tot_subsi(ii)=  (9*keep_subsi(4,ii))**2
     &              + (4*keep_subsi(11,ii))**2
     &              + (12*keep_subsi(19,ii))**2
      enddo

      end

**************************************************************
*
*     combine_for_ZZ_width
*
*  add up results for different channels to calculate ZZ width
*
**************************************************************
      Subroutine combine_for_ZZ_width(av,si,tot_subav,tot_subsi)
      implicit none

      include 'COMMON/max.inc'

      integer run_number,ii
      real*8 av,si,tot_subav(maxsubc),tot_subsi(maxsubc)
      real*8 keep_av(maxfs),keep_si(maxfs)
      real*8 keep_subav(maxfs,maxsubc),keep_subsi(maxfs,maxsubc)
      common/summary_common/keep_av,keep_si,keep_subav,keep_subsi,
     &                      run_number

      av=  3*keep_av(1)
     &   + 3*keep_av(2)
     &   + 9*keep_av(3)
     &   + 3*keep_av(5)
     &   + 3*keep_av(6)
     &   +   keep_av(8)
     &   + 3*keep_av(9)
     &   + 6*keep_av(10)
     &   + 2*keep_av(12)
     &   + 3*keep_av(13)
     &   + 6*keep_av(15)
     &   + 9*keep_av(16)
     &   + 6*keep_av(17)
     &   + 9*keep_av(18)
      si=  (3*keep_si(1))**2
     &   + (3*keep_si(2))**2
     &   + (9*keep_si(3))**2
     &   + (3*keep_si(5))**2
     &   + (3*keep_si(6))**2
     &   + (  keep_si(8))**2
     &   + (3*keep_si(9))**2
     &   + (6*keep_si(10))**2
     &   + (2*keep_si(12))**2
     &   + (3*keep_si(13))**2
     &   + (6*keep_si(15))**2
     &   + (9*keep_si(16))**2
     &   + (6*keep_si(17))**2
     &   + (9*keep_si(18))**2


      do ii=1,maxsubc
      tot_subav(ii)=  3*keep_subav(1,ii)
     &              + 3*keep_subav(2,ii)
     &              + 9*keep_subav(3,ii)
     &              + 3*keep_subav(5,ii)
     &              + 3*keep_subav(6,ii)
     &              + 	keep_subav(8,ii)
     &              + 3*keep_subav(9,ii)
     &              + 6*keep_subav(10,ii)
     &              + 2*keep_subav(12,ii)
     &              + 3*keep_subav(13,ii)
     &              + 6*keep_subav(15,ii)
     &              + 9*keep_subav(16,ii)
     &              + 6*keep_subav(17,ii)
     &              + 9*keep_subav(18,ii)
      tot_subsi(ii)=  (3*keep_subsi(1,ii))**2
     &              + (3*keep_subsi(2,ii))**2
     &              + (9*keep_subsi(3,ii))**2
     &              + (3*keep_subsi(5,ii))**2
     &              + (3*keep_subsi(6,ii))**2
     &              + (  keep_subsi(8,ii))**2
     &              + (3*keep_subsi(9,ii))**2
     &              + (6*keep_subsi(10,ii))**2
     &              + (2*keep_subsi(12,ii))**2
     &              + (3*keep_subsi(13,ii))**2
     &              + (6*keep_subsi(15,ii))**2
     &              + (9*keep_subsi(16,ii))**2
     &              + (6*keep_subsi(17,ii))**2
     &              + (9*keep_subsi(18,ii))**2
      enddo

      end

**************************************************************
*
*     combine_for_Int_width
*
*  add up results for different channels to calculate total width
*
**************************************************************
      Subroutine combine_for_Int_width(av,si,tot_subav,tot_subsi)
      implicit none

      include 'COMMON/max.inc'

      integer run_number,ii
      real*8 av,si,tot_subav(maxsubc),tot_subsi(maxsubc)
      real*8 keep_av(maxfs),keep_si(maxfs)
      real*8 keep_subav(maxfs,maxsubc),keep_subsi(maxfs,maxsubc)
      common/summary_common/keep_av,keep_si,keep_subav,keep_subsi,
     &                      run_number

      av=  3*keep_av(7)
     &   - 3*keep_av(3)
     &   - 3*keep_av(4)
     &   - 2*keep_av(10)
     &   - 2*keep_av(11)
     &   + 2*keep_av(14)
      si=  (3*keep_si(7))**2
     &   + (3*keep_si(3))**2
     &   + (3*keep_si(4))**2
     &   + (2*keep_si(10))**2
     &   + (2*keep_si(11))**2
     &   + (2*keep_si(14))**2

      do ii=1,maxsubc
      tot_subav(ii)=  3*keep_subav(7,ii)
     &              - 3*keep_subav(3,ii)
     &              - 2*keep_subav(4,ii)
     &              - 2*keep_subav(10,ii)
     &              - 2*keep_subav(11,ii)
     &              + 2*keep_subav(14,ii)
      tot_subsi(ii)=  (3*keep_subsi(7,ii))**2
     &              + (3*keep_subsi(3,ii))**2
     &              + (3*keep_subsi(4,ii))**2
     &              + (2*keep_subsi(10,ii))**2
     &              + (2*keep_subsi(11,ii))**2
     &              + (2*keep_subsi(14,ii))**2
      enddo

      end

**************************************************************
*
*     prepare_for_leptonic_width
*
*  setup all channels for calculation of the leptonic width
*
**************************************************************
      subroutine prepare_for_leptonic_width()
      implicit none

      integer whichin
      include 'COMMON/processpar.inc'

      finalstateinput(1)='nue anti-nue num anti-num'
      finalstateinput(2)='e   anti-e   mu  anti-mu'
      finalstateinput(3)='nue anti-nue mu  anti-mu'
      finalstateinput(4)='nue anti-e   mu  anti-num'
      finalstateinput(5)='nue anti-nue nue anti-nue'
      finalstateinput(6)='e   anti-e   e   anti-e'
      finalstateinput(7)='nue anti-e   e   anti-nue'

      do whichin=1,7
        call read_channel(whichin, whichin)
      enddo

      end

**************************************************************
*
*     combine_for_leptonic_width
*
*  add up results for different channels to calculate leptonic width
*
**************************************************************
      Subroutine combine_for_leptonic_width(av,si,tot_subav,tot_subsi)
      implicit none

      include 'COMMON/max.inc'

      integer run_number,ii
      real*8 av,si,tot_subav(maxsubc),tot_subsi(maxsubc)
      real*8 keep_av(maxfs),keep_si(maxfs)
      real*8 keep_subav(maxfs,maxsubc),keep_subsi(maxfs,maxsubc)
      common/summary_common/keep_av,keep_si,keep_subav,keep_subsi,
     &                      run_number

      av=  3*keep_av(1)
     &   + 3*keep_av(2)
     &   + 6*keep_av(3)
     &   + 6*keep_av(4)
     &   + 3*keep_av(5)
     &   + 3*keep_av(6)
     &   + 3*keep_av(7)
      si=  (3*keep_si(1))**2
     &   + (3*keep_si(2))**2
     &   + (6*keep_si(3))**2
     &   + (6*keep_si(4))**2
     &   + (3*keep_si(5))**2
     &   + (3*keep_si(6))**2
     &   + (3*keep_si(7))**2

      do ii=1,maxsubc
      tot_subav(ii)=  3*keep_subav(1,ii)
     &              + 3*keep_subav(2,ii)
     &              + 6*keep_subav(3,ii)
     &              + 6*keep_subav(4,ii)
     &              + 3*keep_subav(5,ii)
     &              + 3*keep_subav(6,ii)
     &              + 3*keep_subav(7,ii)
      tot_subsi(ii)=  (3*keep_subsi(1,ii))**2
     &              + (3*keep_subsi(2,ii))**2
     &              + (6*keep_subsi(3,ii))**2
     &              + (6*keep_subsi(4,ii))**2
     &              + (3*keep_subsi(5,ii))**2
     &              + (3*keep_subsi(6,ii))**2
     &              + (3*keep_subsi(7,ii))**2
      enddo

      end

**************************************************************
*
*     prepare_for_semi_leptonic_width
*
*  setup all channels for calculation of the semi-leptonic width
*
**************************************************************
      subroutine prepare_for_semi_leptonic_width()
      implicit none

      integer whichin
      include 'COMMON/processpar.inc'

      finalstateinput(1)='nue anti-nue uq anti-uq'
      finalstateinput(2)='nue anti-nue dq anti-dq'
      finalstateinput(3)='uq anti-uq e anti-e'
      finalstateinput(4)='dq anti-dq e anti-e'
      finalstateinput(5)='nue anti-e dq anti-uq'

      do whichin=1,5
        call read_channel(whichin, whichin)
      enddo

      end

**************************************************************
*
*     combine_for_semi_leptonic_width
*
*  add up results for different channels to calculate semi-leptonic width
*
**************************************************************
      Subroutine combine_for_semi_leptonic_width(av,si,tot_subav,
     &                                                 tot_subsi)
      implicit none

      include 'COMMON/max.inc'

      integer run_number,ii
      real*8 av,si,tot_subav(maxsubc),tot_subsi(maxsubc)
      real*8 keep_av(maxfs),keep_si(maxfs)
      real*8 keep_subav(maxfs,maxsubc),keep_subsi(maxfs,maxsubc)
      common/summary_common/keep_av,keep_si,keep_subav,keep_subsi,
     &                      run_number

      av=  6*keep_av(1)
     &   + 9*keep_av(2)
     &   + 6*keep_av(3)
     &   + 9*keep_av(4)
     &   + 12*keep_av(5)
      si=  (6*keep_si(1))**2
     &   + (9*keep_si(2))**2
     &   + (6*keep_si(3))**2
     &   + (9*keep_si(4))**2
     &   + (12*keep_si(5))**2

      do ii=1,maxsubc
      tot_subav(ii)=  6*keep_subav(1,ii)
     &              + 9*keep_subav(2,ii)
     &              + 6*keep_subav(3,ii)
     &              + 9*keep_subav(4,ii)
     &              + 12*keep_subav(5,ii)
      tot_subsi(ii)=  (6*keep_subsi(1,ii))**2
     &              + (9*keep_subsi(2,ii))**2
     &              + (6*keep_subsi(3,ii))**2
     &              + (9*keep_subsi(4,ii))**2
     &              + (12*keep_subsi(5,ii))**2
      enddo

      end

**************************************************************
*
*     prepare_for_hadronic_width
*
*  setup all channels for calculation of the hadronic width
*
**************************************************************
      subroutine prepare_for_hadronic_width()
      implicit none

      integer whichin
      include 'COMMON/processpar.inc'

      finalstateinput(1)='uq anti-uq cq anti-cq'
      finalstateinput(2)='dq anti-dq sq anti-sq'
      finalstateinput(3)='uq anti-uq sq anti-sq'
      finalstateinput(4)='uq anti-dq sq anti-cq'
      finalstateinput(5)='uq anti-uq uq anti-uq'
      finalstateinput(6)='dq anti-dq dq anti-dq'
      finalstateinput(7)='uq anti-dq dq anti-uq'

      do whichin=1,7
        call read_channel(whichin, whichin)
      enddo

      end

**************************************************************
*
*     combine_for_hadronic_width
*
*  add up results for different channels to calculate hadronic width
*
**************************************************************
      Subroutine combine_for_hadronic_width(av,si,tot_subav,tot_subsi)
      implicit none

      include 'COMMON/max.inc'

      integer run_number,ii
      real*8 av,si,tot_subav(maxsubc),tot_subsi(maxsubc)
      real*8 keep_av(maxfs),keep_si(maxfs)
      real*8 keep_subav(maxfs,maxsubc),keep_subsi(maxfs,maxsubc)
      common/summary_common/keep_av,keep_si,keep_subav,keep_subsi,
     &                      run_number

      av=    keep_av(1)
     &   + 3*keep_av(2)
     &   + 4*keep_av(3)
     &   + 2*keep_av(4)
     &   + 2*keep_av(5)
     &   + 3*keep_av(6)
     &   + 2*keep_av(7)
      si=  (  keep_si(1))**2
     &   + (3*keep_si(2))**2
     &   + (4*keep_si(3))**2
     &   + (2*keep_si(4))**2
     &   + (2*keep_si(5))**2
     &   + (3*keep_si(6))**2
     &   + (2*keep_si(7))**2

      do ii=1,maxsubc
      tot_subav(ii)=    keep_subav(1,ii)
     &              + 3*keep_subav(2,ii)
     &              + 4*keep_subav(3,ii)
     &              + 2*keep_subav(4,ii)
     &              + 2*keep_subav(5,ii)
     &              + 3*keep_subav(6,ii)
     &              + 2*keep_subav(7,ii)
      tot_subsi(ii)=  (  keep_subsi(1,ii))**2
     &              + (3*keep_subsi(2,ii))**2
     &              + (4*keep_subsi(3,ii))**2
     &              + (2*keep_subsi(4,ii))**2
     &              + (2*keep_subsi(5,ii))**2
     &              + (3*keep_subsi(6,ii))**2
     &              + (2*keep_subsi(7,ii))**2
      enddo

      end

**************************************************************
*
*     prepare_for_WW_width
*
*  setup all channels for calculation of the total width
*
**************************************************************
      subroutine prepare_for_WW_width()
      implicit none

      integer whichin
      include 'COMMON/processpar.inc'

      finalstateinput(1)='nue anti-e   mu  anti-num'
      finalstateinput(2)='uq anti-dq sq anti-cq'
      finalstateinput(3)='nue anti-e dq anti-uq'

      do whichin=1,3
        call read_channel(whichin, whichin)
      enddo

      end

**************************************************************
*
*     combine_for_WW_width_only
*
*  add up results for different channels to calculate WW width
*
**************************************************************
      Subroutine combine_for_WW_width_only(av,si,tot_subav,tot_subsi)
      implicit none

      include 'COMMON/max.inc'

      integer run_number,ii
      real*8 av,si,tot_subav(maxsubc),tot_subsi(maxsubc)
      real*8 keep_av(maxfs),keep_si(maxfs)
      real*8 keep_subav(maxfs,maxsubc),keep_subsi(maxfs,maxsubc)
      common/summary_common/keep_av,keep_si,keep_subav,keep_subsi,
     &                      run_number

      av=  9*keep_av(1)
     &   + 4*keep_av(2)
     &   + 12*keep_av(3)
      si=  (9*keep_si(1))**2
     &   + (4*keep_si(2))**2
     &   + (12*keep_si(3))**2

      do ii=1,maxsubc
      tot_subav(ii)=  9*keep_subav(1,ii)
     &              + 4*keep_subav(2,ii)
     &              + 12*keep_subav(3,ii)
      tot_subsi(ii)=  (9*keep_subsi(1,ii))**2
     &              + (4*keep_subsi(2,ii))**2
     &              + (12*keep_subsi(3,ii))**2
      enddo

      end

**************************************************************
*
*     prepare_for_ZZ_width
*
*  setup all channels for calculation of the total width
*
**************************************************************
      subroutine prepare_for_ZZ_width()
      implicit none

      integer whichin
      include 'COMMON/processpar.inc'

      finalstateinput(1)='nue anti-nue num anti-num'
      finalstateinput(2)='e   anti-e   mu  anti-mu'
      finalstateinput(3)='nue anti-nue mu  anti-mu'
      finalstateinput(4)='nue anti-nue nue anti-nue'
      finalstateinput(5)='e   anti-e   e   anti-e'
      finalstateinput(6)='uq anti-uq cq anti-cq'
      finalstateinput(7)='dq anti-dq sq anti-sq'
      finalstateinput(8)='uq anti-uq sq anti-sq'
      finalstateinput(9)='uq anti-uq uq anti-uq'
      finalstateinput(10)='dq anti-dq dq anti-dq'
      finalstateinput(11)='nue anti-nue uq anti-uq'
      finalstateinput(12)='nue anti-nue dq anti-dq'
      finalstateinput(13)='uq anti-uq e anti-e'
      finalstateinput(14)='dq anti-dq e anti-e'

      do whichin=1,14
        call read_channel(whichin, whichin)
      enddo

      end

**************************************************************
*
*     combine_for_ZZ_width_only
*
*  add up results for different channels to calculate ZZ width
*
**************************************************************
      Subroutine combine_for_ZZ_width_only(av,si,tot_subav,tot_subsi)
      implicit none

      include 'COMMON/max.inc'

      integer run_number,ii
      real*8 av,si,tot_subav(maxsubc),tot_subsi(maxsubc)
      real*8 keep_av(maxfs),keep_si(maxfs)
      real*8 keep_subav(maxfs,maxsubc),keep_subsi(maxfs,maxsubc)
      common/summary_common/keep_av,keep_si,keep_subav,keep_subsi,
     &                      run_number

      av=  3*keep_av(1)
     &   + 3*keep_av(2)
     &   + 9*keep_av(3)
     &   + 3*keep_av(4)
     &   + 3*keep_av(5)
     &   +   keep_av(6)
     &   + 3*keep_av(7)
     &   + 6*keep_av(8)
     &   + 2*keep_av(9)
     &   + 3*keep_av(10)
     &   + 6*keep_av(11)
     &   + 9*keep_av(12)
     &   + 6*keep_av(13)
     &   + 9*keep_av(14)
      si=  (3*keep_si(1))**2
     &   + (3*keep_si(2))**2
     &   + (9*keep_si(3))**2
     &   + (3*keep_si(4))**2
     &   + (3*keep_si(5))**2
     &   + (  keep_si(6))**2
     &   + (3*keep_si(7))**2
     &   + (6*keep_si(8))**2
     &   + (2*keep_si(9))**2
     &   + (3*keep_si(10))**2
     &   + (6*keep_si(11))**2
     &   + (9*keep_si(12))**2
     &   + (6*keep_si(13))**2
     &   + (9*keep_si(14))**2


      do ii=1,maxsubc
      tot_subav(ii)=  3*keep_subav(1,ii)
     &              + 3*keep_subav(2,ii)
     &              + 9*keep_subav(3,ii)
     &              + 3*keep_subav(4,ii)
     &              + 3*keep_subav(5,ii)
     &              + 	keep_subav(6,ii)
     &              + 3*keep_subav(7,ii)
     &              + 6*keep_subav(8,ii)
     &              + 2*keep_subav(9,ii)
     &              + 3*keep_subav(10,ii)
     &              + 6*keep_subav(11,ii)
     &              + 9*keep_subav(12,ii)
     &              + 6*keep_subav(13,ii)
     &              + 9*keep_subav(14,ii)
      tot_subsi(ii)=  (3*keep_subsi(1,ii))**2
     &              + (3*keep_subsi(2,ii))**2
     &              + (9*keep_subsi(3,ii))**2
     &              + (3*keep_subsi(4,ii))**2
     &              + (3*keep_subsi(5,ii))**2
     &              + (  keep_subsi(6,ii))**2
     &              + (3*keep_subsi(7,ii))**2
     &              + (6*keep_subsi(8,ii))**2
     &              + (2*keep_subsi(9,ii))**2
     &              + (3*keep_subsi(10,ii))**2
     &              + (6*keep_subsi(11,ii))**2
     &              + (9*keep_subsi(12,ii))**2
     &              + (6*keep_subsi(13,ii))**2
     &              + (9*keep_subsi(14,ii))**2
      enddo

      end

**************************************************************
*
*     prepare_for_Int_width
*
*  setup all channels for calculation of the total width
*
**************************************************************
      subroutine prepare_for_Int_width()
      implicit none

      integer whichin
      include 'COMMON/processpar.inc'

      finalstateinput(1)='nue anti-e   e   anti-nue'
      finalstateinput(2)='nue anti-nue mu  anti-mu'
      finalstateinput(3)='nue anti-e   mu  anti-num'
      finalstateinput(4)='uq anti-uq sq anti-sq'
      finalstateinput(5)='uq anti-dq sq anti-cq'
      finalstateinput(6)='uq anti-dq dq anti-uq'

      do whichin=1,6
        call read_channel(whichin, whichin)
      enddo

      end

**************************************************************
*
*     combine_for_Int_width_only
*
*  add up results for different channels to calculate total width
*
**************************************************************
      Subroutine combine_for_Int_width_only(av,si,tot_subav,tot_subsi)
      implicit none

      include 'COMMON/max.inc'

      integer run_number,ii
      real*8 av,si,tot_subav(maxsubc),tot_subsi(maxsubc)
      real*8 keep_av(maxfs),keep_si(maxfs)
      real*8 keep_subav(maxfs,maxsubc),keep_subsi(maxfs,maxsubc)
      common/summary_common/keep_av,keep_si,keep_subav,keep_subsi,
     &                      run_number

      av=  3*keep_av(1)
     &   - 3*keep_av(2)
     &   - 3*keep_av(3)
     &   - 2*keep_av(4)
     &   - 2*keep_av(5)
     &   + 2*keep_av(6)
      si=  (3*keep_si(1))**2
     &   + (3*keep_si(2))**2
     &   + (3*keep_si(3))**2
     &   + (2*keep_si(4))**2
     &   + (2*keep_si(5))**2
     &   + (2*keep_si(6))**2

      do ii=1,maxsubc
      tot_subav(ii)=  3*keep_subav(1,ii)
     &              - 3*keep_subav(2,ii)
     &              - 2*keep_subav(3,ii)
     &              - 2*keep_subav(4,ii)
     &              - 2*keep_subav(5,ii)
     &              + 2*keep_subav(6,ii)
      tot_subsi(ii)=  (3*keep_subsi(1,ii))**2
     &              + (3*keep_subsi(2,ii))**2
     &              + (3*keep_subsi(3,ii))**2
     &              + (2*keep_subsi(4,ii))**2
     &              + (2*keep_subsi(5,ii))**2
     &              + (2*keep_subsi(6,ii))**2
      enddo

      end


**************************************************************
*
*     write_summary
*
*  write output to summarize results
*
**************************************************************
      Subroutine write_summary(av,si,tot_subav,tot_subsi,nout)
      implicit none

      include 'COMMON/max.inc'
C      include 'COMMON/output.inc'
      include 'COMMON/options.inc'

      integer nout
      integer run_number,ii
      real*8 av,si,tot_subav(maxsubc),tot_subsi(maxsubc)
      real*8 keep_av(maxfs),keep_si(maxfs)
      real*8 keep_subav(maxfs,maxsubc),keep_subsi(maxfs,maxsubc)
      common/summary_common/keep_av,keep_si,keep_subav,keep_subsi,
     &                      run_number


      write(nout,1000)'  Average             = ',av
      write(nout,1000)'  Standard deviation  = ',sqrt(si)
      write(nout,*)
      write(nout,*)' Subcontributions:'
      write(nout,*)' -----------------'
      if (qwidth.eq.0) then
         write(nout,*)' Born cross section (LO gauge-',
     &        'boson width):'
      else
         write(nout,*)' Born cross section (NLO gauge-',
     &        'boson width):'
      endif
      write(nout,1000)'   Average            = ',tot_subav(1)
      write(nout,1000)'   Standard deviation = ',sqrt(tot_subsi(1))
      if (tot_subav(2).ne.0d0 .and. tot_subav(4).ne.0d0) then
        write(nout,*)
        write(nout,*)' EW corrections:'
        write(nout,1000)'   Average            = ',
     &       tot_subav(2) + tot_subav(4)
        write(nout,1000)'   Standard deviation = ',
     &       dsqrt(tot_subsi(2) + tot_subsi(4))
      endif
      if (tot_subav(3).ne.0d0 .and. tot_subav(5).ne.0d0) then
        write(nout,*)
        write(nout,*)' QCD corrections:'
        write(nout,1000)'   Average            = ',
     &       tot_subav(3) + tot_subav(5)
        write(nout,1000)'   Standard deviation = ',
     &       dsqrt(tot_subsi(3) + tot_subsi(5))
      endif

 1000 format((a),f18.10," MeV")

      end
